from contextlib import suppress
from random import choice
from tempfile import TemporaryDirectory

from twisted.cred import portal, credentials
from twisted.cred.checkers import ICredentialsChecker
from twisted.cred.error import UnauthorizedLogin, UnhandledCredentials
from twisted.cred.portal import Portal
from twisted.internet import reactor, defer
from twisted.protocols.ftp import (
    FTPAnonymousShell,
    FTPFactory,
    FTP,
    IFTPShell,
    GUEST_LOGGED_IN_PROCEED,
    AuthorizationError,
    USR_LOGGED_IN_PROCEED,
)
from twisted.python import filepath
from zope.interface import implementer

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    check_bytes,
)


class QFTPServer(BaseServer):
    NAME = "ftp_server"
    DEFAULT_PORT = 21

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.mocking_server = choice(
            [
                "ProFTPD 1.2.10",
                "ProFTPD 1.3.4a",
                "FileZilla ftp 0.9.43",
                "Gene6 ftpd 3.10.0",
                "FileZilla ftp 0.9.33",
                "ProFTPD 1.2.8",
            ]
        )
        self.temp_folder = TemporaryDirectory()

    def server_main(self):  # noqa: C901
        _q_s = self

        @implementer(portal.IRealm)
        class CustomFTPRealm:
            def __init__(self, anonymousRoot):  # noqa: N803
                self.anonymousRoot = filepath.FilePath(anonymousRoot)

            def requestAvatar(  # noqa: N802
                self,
                avatarId,  # noqa: ARG002,N803
                mind,  # noqa: ARG002
                *interfaces,
            ):
                for iface in interfaces:
                    if iface is IFTPShell:
                        avatar = FTPAnonymousShell(self.anonymousRoot)
                        return (
                            IFTPShell,
                            avatar,
                            getattr(avatar, "logout", lambda: None),
                        )
                raise NotImplementedError("Only IFTPShell interface is supported by this realm")

        @implementer(ICredentialsChecker)
        class CustomAccess:
            credentialInterfaces = (  # noqa: N815
                credentials.IAnonymous,
                credentials.IUsernamePassword,
            )

            def requestAvatarId(self, credentials):  # noqa: N802
                with suppress(Exception):
                    username = check_bytes(credentials.username)
                    password = check_bytes(credentials.password)
                    if username == _q_s.username and password == _q_s.password:
                        return defer.succeed(credentials.username)
                return defer.fail(UnauthorizedLogin())

        class CustomFTPProtocol(FTP):
            def connectionMade(self):  # noqa: N802
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )
                self.state = self.UNAUTH
                self.setTimeout(self.timeOut)
                self.reply("220.2", self.factory.welcomeMessage)

            def processCommand(self, cmd, *params):  # noqa: N802
                with suppress(Exception):
                    if "capture_commands" in _q_s.options:
                        _q_s.log(
                            {
                                "action": "command",
                                "data": {
                                    "cmd": check_bytes(cmd.upper()),
                                    "args": check_bytes(params),
                                },
                                "src_ip": self.transport.getPeer().host,
                                "src_port": self.transport.getPeer().port,
                            }
                        )
                return super().processCommand(cmd, *params)

            def ftp_PASS(self, password):  # noqa: N802
                username = check_bytes(self._user)
                password = check_bytes(password)
                peer = self.transport.getPeer()
                _q_s.check_login(username, password, ip=peer.host, port=peer.port)

                if self.factory.allowAnonymous and self._user == self.factory.userAnonymous:
                    creds = credentials.Anonymous()
                    reply = GUEST_LOGGED_IN_PROCEED
                else:
                    creds = credentials.UsernamePassword(self._user, password)
                    reply = USR_LOGGED_IN_PROCEED

                del self._user

                def _cbLogin(parsed):  # noqa: N802
                    self.shell = parsed[1]
                    self.logout = parsed[2]
                    self.workingDirectory = []
                    self.state = self.AUTHED
                    return reply

                def _ebLogin(failure):  # noqa: N802
                    failure.trap(UnauthorizedLogin, UnhandledCredentials)
                    self.state = self.UNAUTH
                    raise AuthorizationError

                d = self.portal.login(creds, None, IFTPShell)
                d.addCallbacks(_cbLogin, _ebLogin)
                return d

        p = Portal(CustomFTPRealm("data"), [CustomAccess()])
        factory = FTPFactory(p)
        factory.protocol = CustomFTPProtocol
        factory.welcomeMessage = "ProFTPD 1.2.10"
        reactor.listenTCP(port=self.port, factory=factory)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from ftplib import FTP as FFTP

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password
            f = FFTP()
            f.connect(_ip, _port)
            f.login(_username, _password)
            f.pwd()
            f.quit()


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        ftpserver = QFTPServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        ftpserver.run_server()



from __future__ import annotations

from contextlib import suppress
from hashlib import sha1
from pathlib import Path
from struct import pack

from twisted.internet import reactor
from twisted.internet.protocol import Factory, Protocol

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    check_bytes,
)


class QMysqlServer(BaseServer):
    NAME = "mysql_server"
    DEFAULT_PORT = 3306

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        if hasattr(self, "file_name"):
            self.words = Path(self.file_name).read_text("utf-8").splitlines()
        else:
            self.words = [self.password.encode()]

    def greeting(self):
        base = [
            b"\x0a",
            b"5.7.00" + b"\0",
            b"\x36\x00\x00\x00",
            b"12345678" + b"\0",
            b"\xff\xf7",
            b"\x21",
            b"\x02\x00",
            b"\x0f\x81",
            b"\x15",
            b"\0" * 10,
            b"123456789012" + b"\0",
            b"mysql_native_password" + b"\0",
        ]
        return self._create_response(base)

    def too_many(self):
        base = [b"\xff", b"\x10\x04", b"#08004", b"Too many connections"]
        return self._create_response(base, sequence_id=2)

    def access_denied(self):
        base = [b"\xff", b"\x15\x04", b"#28000", b"Access denied.."]
        return self._create_response(base, sequence_id=2)

    @staticmethod
    def _create_response(base: list[bytes], sequence_id: int = 0) -> bytes:
        # MySQL Packets structure: 3 byte payload_len, 1 byte sequence_id, payload
        payload_len = pack("<I", len(b"".join(base)))[:3]
        payload = b"".join(base)
        return payload_len + pack("b", sequence_id) + payload

    @staticmethod
    def parse_data(data: bytes) -> tuple[bytes, bytes, bool]:
        username, password = "", ""
        username_offset = 36
        username_len = data[username_offset:].find(b"\x00")
        if username_len != -1:
            with suppress(IndexError):
                username = data[username_offset : username_offset + username_len]
                password_offset = username_offset + username_len + 2
                password_len = data[password_offset - 1]
                password = data[password_offset : password_offset + password_len]
                if password_len == 20:  # noqa: PLR2004
                    return username, password, True
        return username, password, False

    def decode(self, hash_: bytes):
        with suppress(Exception):
            for word in self.words:
                hash1 = sha1(word.strip(b"\n")).digest()
                hash2 = sha1(hash1).digest()
                encrypted = [
                    (a ^ b) for a, b in zip(hash1, sha1(b"12345678123456789012" + hash2).digest())
                ]
                if encrypted == list(hash_):
                    return word
        return None

    def server_main(self):
        _q_s = self

        class CustomMysqlProtocol(Protocol):
            _state = None

            def connectionMade(self):  # noqa: N802
                self._state = 1
                self.transport.write(_q_s.greeting())
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )

            def dataReceived(self, data):  # noqa: N802
                try:
                    if self._state == 1:
                        ret_access_denied = False
                        username, password, good = _q_s.parse_data(data)
                        username = check_bytes(username)
                        status = "failed"
                        if good:
                            if password:
                                password_decoded = _q_s.decode(password)
                                if password_decoded is not None and username == _q_s.username:
                                    password = check_bytes(password_decoded)
                                    status = "success"
                                else:
                                    password = password.hex()
                                    ret_access_denied = True
                            else:
                                ret_access_denied = True
                                password = ":".join(hex(c)[2:] for c in data)
                        _q_s.log(
                            {
                                "action": "login",
                                "status": status,
                                "src_ip": self.transport.getPeer().host,
                                "src_port": self.transport.getPeer().port,
                                "username": username,
                                "password": password,
                            }
                        )

                        if ret_access_denied:
                            self.transport.write(_q_s.access_denied())
                        else:
                            self.transport.write(_q_s.too_many())
                    else:
                        self.transport.loseConnection()
                except BaseException:
                    self.transport.write(_q_s.too_many())
                    self.transport.loseConnection()

            def connectionLost(self, reason):  # noqa: N802,ARG002
                self._state = None

        factory = Factory()
        factory.protocol = CustomMysqlProtocol
        reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from mysql.connector import connect as mysqlconnect

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password
            mysqlconnect(
                user=_username,
                password=_password,
                host=_ip,
                port=_port,
                database="test",
                connect_timeout=1000,
            )


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qmysqlserver = QMysqlServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        qmysqlserver.run_server()



from __future__ import annotations

import pytest
from dns.resolver import Resolver

from honeypots import QDNSServer
from .utils import (
    assert_connect_is_logged,
    IP,
    load_logs_from_file,
    wait_for_server,
)

PORT = "50053"


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QDNSServer, "port": PORT}],
    indirect=True,
)
def test_dns_server(server_logs):
    with wait_for_server(PORT):
        resolver = Resolver(configure=False)
        resolver.nameservers = [IP]
        resolver.port = int(PORT)
        domain = "example.org"
        responses = [
            resolver.resolve(domain, "a", tcp=False),
            resolver.resolve(domain, "a", tcp=True),
        ]

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 3
    connect, *queries = logs
    assert_connect_is_logged(connect, PORT)

    for query in queries:
        assert query["action"] == "query"
        assert "data" in query
        assert "<A address=" in query["data"]

    for response in responses:
        assert domain in response.canonical_name.to_text()



from __future__ import annotations

import logging
from _thread import start_new_thread
from binascii import hexlify
from contextlib import suppress
from datetime import datetime
from io import StringIO
from random import choice
import re
from socket import socket, AF_INET, SOCK_STREAM, SOL_SOCKET, SO_REUSEADDR
from threading import Event
from time import time
from typing import TYPE_CHECKING

from paramiko import (
    RSAKey,
    ServerInterface,
    Transport,
)
from paramiko.common import (
    AUTH_FAILED,
    AUTH_SUCCESSFUL,
    OPEN_FAILED_ADMINISTRATIVELY_PROHIBITED,
    OPEN_SUCCEEDED,
)
from paramiko.ssh_exception import SSHException

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    check_bytes,
)

if TYPE_CHECKING:
    from paramiko.channel import Channel


# deactivate logging output of paramiko
logging.getLogger("paramiko").setLevel(logging.CRITICAL)

CTRL_C = b"\x03"
CTRL_D = b"\x04"
ANSI_SEQUENCE = b"\x1b"
DEL = b"\x7f"
COMMANDS = {
    "ls": (
        "bin boot cdrom dev etc home lib lib32 libx32 lib64 lost+found media mnt opt proc root "
        "run sbin snap srv sys tmp usr var"
    ),
    "pwd": "/",
    "whoami": "root",
    "": "",
    "cd": "",
    "cd /": "",
    "uname": "Linux",
    "uname -s": "Linux",
    "uname -n": "n1-v26",
    "uname -r": "5.4.0-26-generic",
    "uname -v": "#26-Ubuntu SMP %TIME",
    "uname -m": "x86_64",
    "uname -p": "x86_64",
    "uname -i": "x86_64",
    "uname -o": "GNU/Linux",
    "uname -a": (
        "Linux n1-v26 5.4.0-26-generic #26-Ubuntu SMP %TIME x86_64 x86_64 x86_64 GNU/Linux"
    ),
}
ANSI_REGEX = re.compile(rb"(?:\x1B[@-_]|[\x80-\x9F])[0-?]*[ -/]*[@-~]")


class QSSHServer(BaseServer):
    NAME = "ssh_server"
    DEFAULT_PORT = 22

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.mocking_server = choice(
            ["OpenSSH 7.5", "OpenSSH 7.3", "Serv-U SSH Server 15.1.1.108", "OpenSSH 6.4"]
        )

    @staticmethod
    def generate_pub_pri_keys() -> str:
        key = RSAKey.generate(2048)
        string_io = StringIO()
        key.write_private_key(string_io)
        return string_io.getvalue()

    def server_main(self):  # noqa: C901
        _q_s = self

        class SSHHandle(ServerInterface):
            def __init__(self, ip, port):
                self.ip = ip
                self.port = port
                self.event = Event()

            def check_channel_request(self, kind, *_, **__):
                if kind == "session":
                    return OPEN_SUCCEEDED
                return OPEN_FAILED_ADMINISTRATIVELY_PROHIBITED

            def check_auth_password(self, username, password):
                username = check_bytes(username)
                password = check_bytes(password)
                if _q_s.check_login(username, password, self.ip, self.port):
                    return AUTH_SUCCESSFUL
                return AUTH_FAILED

            def check_channel_exec_request(self, channel, command):  # noqa: ARG002
                if "capture_commands" in _q_s.options:
                    _q_s.log(
                        {
                            "action": "command",
                            "src_ip": self.ip,
                            "src_port": self.port,
                            "data": {"command": check_bytes(command)},
                        }
                    )
                self.event.set()
                return True

            def get_allowed_auths(self, *_, **__):
                return "password,publickey"

            def check_auth_publickey(self, username, key):
                _q_s.log(
                    {
                        "action": "login",
                        "src_ip": self.ip,
                        "src_port": self.port,
                        "username": check_bytes(username),
                        "key_fingerprint": check_bytes(hexlify(key.get_fingerprint())),
                    }
                )
                return AUTH_SUCCESSFUL

            def check_channel_shell_request(self, *_, **__):
                return True

            def check_channel_direct_tcpip_request(self, *_, **__):
                return OPEN_SUCCEEDED

            def check_channel_pty_request(self, *_, **__):
                return True

        def handle_connection(client, priv):
            try:
                ip, port = client.getpeername()
            except OSError as err:
                _q_s.logger.debug(f"Server error: {err}")
                return
            _q_s.log(
                {
                    "action": "connection",
                    "src_ip": ip,
                    "src_port": port,
                }
            )

            with Transport(client) as session:
                session.local_version = f"SSH-2.0-{_q_s.mocking_server}"
                session.add_server_key(RSAKey(file_obj=StringIO(priv)))
                ssh_handle = SSHHandle(ip, port)
                try:
                    session.start_server(server=ssh_handle)
                except (SSHException, EOFError, ConnectionResetError) as err:
                    _q_s.logger.debug(f"Server error: {err}", exc_info=True)
                    return

                with session.accept(30) as conn:
                    if "interactive" in _q_s.options and conn is not None:
                        _handle_interactive_session(conn, ip, port)
                    with suppress(TimeoutError):
                        ssh_handle.event.wait(2)

        def _handle_interactive_session(conn: Channel, ip: str, port: int):
            conn.send(b"Welcome to Ubuntu 20.04 LTS (GNU/Linux 5.4.0-26-generic x86_64)\r\n\r\n")
            timeout = time() + 300
            while time() < timeout:
                try:
                    conn.send(b"$ ")
                    line = _receive_line(conn)
                except (TimeoutError, EOFError):
                    break
                _q_s.log(
                    {
                        "action": "interactive",
                        "src_ip": ip,
                        "src_port": port,
                        "data": {"command": line},
                    }
                )
                if line == "exit":
                    break
                _respond(conn, line)

        sock = socket(AF_INET, SOCK_STREAM)
        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
        sock.bind((self.ip, self.port))
        sock.listen(1)
        private_key = self.generate_pub_pri_keys()
        while True:
            with suppress(Exception):
                client, _ = sock.accept()
                start_new_thread(handle_connection, (client, private_key))

    def test_server(self, ip=None, port=None, username=None, password=None):
        from paramiko import SSHClient, AutoAddPolicy

        _ip = ip or self.ip
        _port = port or self.port
        _username = username or self.username
        _password = password or self.password
        ssh = SSHClient()
        ssh.set_missing_host_key_policy(
            AutoAddPolicy()
        )  # if you have default ones, remove them before using this..
        ssh.connect(_ip, port=_port, username=_username, password=_password)


def _receive_line(conn: Channel) -> str:
    line = b""
    while not any(line.endswith(char) for char in [b"\r", b"\n", CTRL_C]):
        # timeout if the user does not send anything for 10 seconds
        conn.settimeout(10)
        # a button press may equate to multiple bytes (e.g. non-ascii chars,
        # ANSI sequences, etc.), so we receive more than one byte here
        recv = conn.recv(1024)
        if not recv or recv == CTRL_D:  # capture ctrl+D
            conn.send(b"^D\r\n")
            raise EOFError
        if recv == CTRL_C:
            conn.send(b"^C\r\n")
        elif recv == b"\r":
            # ssh only sends "\r" on enter press so we also need to send "\n" back
            conn.send(b"\n")
        elif ANSI_SEQUENCE in recv:
            recv = ANSI_REGEX.sub(b"", recv)
        if DEL in recv:
            recv.replace(DEL, b"")
        if recv:
            line += recv
            conn.send(recv)
    return line.strip().decode(errors="replace")


def _respond(conn: Channel, line: str):
    if line == "" or line.endswith(CTRL_C.decode()):
        return
    if line in COMMANDS:
        response = COMMANDS.get(line)
        if "%TIME" in response:
            response = response.replace(
                "%TIME", datetime.now().strftime("%a %b %d %H:%M:%S UTC %Y")
            )
        conn.send(f"{response}\r\n".encode())
    elif line.startswith("cd "):
        target = _parse_args(line)
        if not target:
            conn.send(b"\r\n")
        else:
            if target.startswith("~"):
                target = target.replace("~", "/root")
            conn.send(f"sh: 1: cd: can't cd to {target}\r\n".encode())
    elif line.startswith("ls "):
        target = _parse_args(line)
        if not target:
            conn.send(f"{COMMANDS['ls']}\r\n".encode())
        else:
            conn.send(f"ls: cannot open directory '{target}': Permission denied\r\n".encode())
    else:
        conn.send(f"{line}: command not found\r\n".encode())


def _parse_args(line: str) -> str | None:
    args = [i for i in line.split(" ")[1:] if i and not i.startswith("-")]
    if args:
        return args[0]
    return None


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qsshserver = QSSHServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        qsshserver.run_server()



from __future__ import annotations

from base64 import b64decode
from smtplib import SMTP

import pytest

from honeypots import QSMTPServer
from .utils import (
    assert_connect_is_logged,
    assert_login_is_logged,
    IP,
    load_logs_from_file,
    PASSWORD,
    USERNAME,
    wait_for_server,
)

PORT = "50025"
SERVER_CONFIG = {
    "honeypots": {
        "smtp": {
            "options": ["capture_commands"],
        },
    }
}


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QSMTPServer, "port": PORT, "custom_config": SERVER_CONFIG}],
    indirect=True,
)
def test_smtp_server(server_logs):
    with wait_for_server(PORT):
        client = SMTP(IP, int(PORT))
        client.ehlo()
        client.login(USERNAME, PASSWORD)
        client.quit()

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 4
    connect1, auth, login, *additional = logs
    assert_connect_is_logged(connect1, PORT)
    assert_login_is_logged(login)

    assert auth["data"]["command"] == "AUTH"
    assert b64decode(auth["data"]["data"]).decode() == f"\x00{USERNAME}\x00{PASSWORD}"




from __future__ import annotations

import pytest

from honeypots import QRDPServer
from .utils import (
    assert_connect_is_logged,
    connect_to,
    IP,
    load_logs_from_file,
    wait_for_server,
)

PORT = "53389"


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QRDPServer, "port": PORT}],
    indirect=True,
)
def test_rdp_server(server_logs):
    with wait_for_server(PORT), connect_to(IP, PORT) as connection:
        connection.send(b"test")
        connection.send(
            b"\x03\x00\x00*%\xe0\x00\x00\x00\x00\x00Cookie: "
            b"mstshash=foobar\r\n\x01\x00\x08\x00\x03\x00\x00"
        )

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 2
    connect, stshash = logs
    assert_connect_is_logged(connect, PORT)

    assert stshash["action"] == "stshash"
    assert stshash["mstshash"] == "success"
    assert "stshash" in stshash["data"]
    assert "foobar" in stshash["data"]["stshash"]



from __future__ import annotations

from contextlib import suppress
from re import findall

from twisted.internet import reactor
from twisted.internet.protocol import Factory, Protocol

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
)


class QOracleServer(BaseServer):
    NAME = "oracle_server"
    DEFAULT_PORT = 1521

    def server_main(self):  # noqa: C901
        _q_s = self

        class CustomRedisProtocol(Protocol):
            _state = None

            @staticmethod
            def wrong_password() -> bytes:
                return (
                    b"\x02B\xc5\xbb\xe7\x7f\x02B\xac\x11\x00\x02\x08\x00E\x00\x01\x02Md@\x00@\x06"
                    b"\x94l\xac\x11\x00\x02\xac\x11\x00\x01\x05\xf1\xa5\xa8\xab\xf5\xff\x94\x98"
                    b"\xdf\xd5\xa1\x80\x18\x01\xf5Y\x1a\x00\x00\x01\x01\x08\nJ\xe7\xf0,\xb2,\xfe"
                    b"\x08\x00\x00\x00\xce\x06\x00\x00\x00\x00\x00\x04\x01\x00\x00\x00\x00\x00"
                    b"\x01\x00\x00\x00\x00\xf9\x03\x00\x00\x00\x00\x00\x00\x00\x00\x02\x00\x00"
                    b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    b"\x00\x00\x00\x00\x00\x00\x03\x00\x00\x00\x00\x00\x006\x01\x00\x00\x00\x00"
                    b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00 \x82\x1c\x86u\x7f\x00\x00\x00\x00"
                    b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                    b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xf9\x03\x00\x00"
                    b"\x00\x00\x00\x00\x00\x00\x00\x003ORA-01017: "
                    b"invalid username/password; logon denied\n"
                )

            @staticmethod
            def refuse_payload() -> bytes:
                return b"\x00\x08\x00\x00\x04\x00\x00\x00"

            @staticmethod
            def parse_payload(data: bytes) -> tuple[str | None, str | None, str | None]:
                service_name = None
                program = None
                local_user = None
                with suppress(Exception):
                    offset = data.index(b"(DESCRIPTION=")
                    connect = data[offset:].split(b"\0")[0]
                    attributes = [
                        item for item in findall(rb"[^\(\)]+", connect) if not item.endswith(b"=")
                    ]
                    for item in attributes:
                        name, value = item.split(b"=", maxsplit=1)
                        if name.startswith(b"SERVICE_NAME"):
                            service_name = value.decode()
                        elif name.startswith(b"PROGRAM"):
                            program = value.decode()
                        elif name.startswith(b"USER"):
                            local_user = value.decode()
                return service_name, program, local_user

            def connectionMade(self):  # noqa: N802
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )

            def dataReceived(self, data: bytes):  # noqa: N802
                service_name, program, local_user = self.parse_payload(data)
                if service_name or program or local_user:
                    _q_s.log(
                        {
                            "action": "login",
                            "src_ip": self.transport.getPeer().host,
                            "src_port": self.transport.getPeer().port,
                            "data": {
                                "local_user": local_user,
                                "program": program,
                                "service_name": service_name,
                            },
                        }
                    )
                self.transport.write(self.refuse_payload())
                self.transport.loseConnection()

        factory = Factory()
        factory.protocol = CustomRedisProtocol
        reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):  # noqa: ARG002
        with suppress(Exception):
            from warnings import filterwarnings

            filterwarnings(action="ignore", module=".*socket.*")
            from socket import socket, AF_INET, SOCK_STREAM

            payload = (
                b"\x00\x00\x03\x04\x00\x06\x00\x00\x00\x00\x00\x00\x00\x00\x08\x00E\x00\x01F\xb9"
                b"\xd9@\x00@\x06\x81\xd6\x7f\x00\x00\x01\x7f\x00\x00\x01\xbf\xce\x06\x13\xacW\xde"
                b"\xc0Z\xb5\x0cI\x80\x18\x02\x00\xff:\x00\x00\x01\x01\x08\n\x1bdZ^\x1bdZ^\x01\x12"
                b"\x00\x00\x01\x00\x00\x00\x01>\x01,\x0cA \x00\xff\xff\x7f\x08\x00\x00\x01\x00"
                b"\x00\xc8\x00J\x00\x00\x14\x00AA\xa7C\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00 \x00\x00 \x00\x00\x00"
                b"\x00\x00\x00\x00\x00\x00\x01(DESCRIPTION=(CONNECT_DATA=(SERVICE_NAME=xe)"
                b"(CID=(PROGRAM=linux_1)(HOST=xxxxxxxxxxxxxx)(USER=xxxxxxxxxxxxxx))"
                b"(CONNECTION_ID=xxxxxxxxxxxxxxxxxxxxxxxx))(ADDRESS=(PROTOCOL=tcp)"
                b"(HOST=xxxxxxx)(PORT=xxxx)))"
            )
            _ip = ip or self.ip
            _port = port or self.port
            c = socket(AF_INET, SOCK_STREAM)
            c.connect((_ip, _port))
            c.send(payload)
            data, address = c.recvfrom(10000)
            c.close()


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        QOracleServer = QOracleServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        QOracleServer.run_server()



from __future__ import annotations

import pytest

from honeypots import QDHCPServer
from .utils import (
    connect_to,
    EXPECTED_KEYS,
    IP,
    load_logs_from_file,
    wait_for_server,
)

PORT = "50067"


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QDHCPServer, "port": PORT}],
    indirect=True,
)
def test_dhcp_server(server_logs):
    with wait_for_server(PORT), connect_to(IP, PORT, udp=True) as connection:
        connection.send(b"\x03" * 240)

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 1
    (query,) = logs
    assert all(k in query for k in EXPECTED_KEYS)
    assert query["action"] == "query"
    assert query["status"] == "success"
    assert query["data"] == {"mac_address": "03:03:03:03:03:03"}



from __future__ import annotations

from contextlib import suppress
from email.parser import BytesParser
from pathlib import Path

from dns.resolver import resolve as dsnquery
from twisted.internet import reactor
from twisted.internet.protocol import Protocol, Factory

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    load_template,
)

DUMMY_TEMPLATE = load_template("dummy_page.html")


class QHTTPProxyServer(BaseServer):
    NAME = "http_proxy_server"
    DEFAULT_PORT = 8080

    def __init__(self, **kwargs):
        self.template: str | None = None
        super().__init__(**kwargs)
        self.template_contents: str | None = self._load_template()

    def _load_template(self) -> str | None:
        if self.template:
            try:
                template_contents = Path(self.template).read_text(errors="replace")
                self.logger.debug(
                    f"[{self.NAME}]: Successfully loaded custom template {self.template}"
                )
                return template_contents
            except FileNotFoundError:
                self.logger.error(f"[{self.NAME}]: Template file {self.template} not found")
        return None

    def server_main(self):
        _q_s = self

        class CustomProtocolParent(Protocol):
            def __init__(self):
                self.buffer = None
                self.client = None

            def resolve_domain(self, request_string):
                with suppress(Exception):
                    _, parsed_request = request_string.split(b"\r\n", 1)
                    headers = BytesParser().parsebytes(parsed_request)
                    host = headers["host"].split(":")
                    _q_s.log(
                        {
                            "action": "query",
                            "src_ip": self.transport.getPeer().host,
                            "src_port": self.transport.getPeer().port,
                            "data": host[0],
                        }
                    )
                    return dsnquery(host[0], "A")[0].address
                return None

            def dataReceived(self, data):  # noqa: N802
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )
                ip = self.resolve_domain(data)
                if ip:
                    self.write(_create_dummy_response(_q_s.template_contents or DUMMY_TEMPLATE))
                else:
                    self.transport.loseConnection()

                if self.client:
                    self.client.write(data)
                else:
                    self.buffer = data

            def write(self, data):
                self.transport.write(data)

        factory = Factory()
        factory.protocol = CustomProtocolParent
        reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, domain=None):
        with suppress(Exception):
            from requests import get

            _ip = ip or self.ip
            _port = port or self.port
            _domain = domain or "http://yahoo.com"
            get(_domain, proxies={"http": f"http://{_ip}:{_port}"}).text.encode("ascii", "ignore")


def _create_dummy_response(content: str) -> bytes:
    response = [
        "HTTP/1.1 200 OK",
        f"Content-Length: {len(content)}",
        "",
        f"{content}",
    ]
    return "\r\n".join(response).encode()


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qhttpproxyserver = QHTTPProxyServer(
            ip=parsed.ip, port=parsed.port, options=parsed.options, config=parsed.config
        )
        qhttpproxyserver.run_server()



from __future__ import annotations

from contextlib import suppress

from twisted.internet import reactor
from twisted.internet.protocol import Factory
from twisted.mail.pop3 import POP3, POP3Error

from honeypots.base_server import BaseServer
from honeypots.helper import check_bytes, server_arguments


class QPOP3Server(BaseServer):
    NAME = "pop3_server"
    DEFAULT_PORT = 110

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.mocking_server = "Microsoft Exchange POP3 service is ready"

    def server_main(self):  # noqa: C901
        _q_s = self

        class CustomPOP3Protocol(POP3):
            self._user = None

            def connectionMade(self):  # noqa: N802
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )
                self._user = None
                self.successResponse(_q_s.mocking_server)

            def processCommand(self, command: bytes, *args):  # noqa: N802
                with suppress(Exception):
                    if "capture_commands" in _q_s.options:
                        _q_s.log(
                            {
                                "action": "command",
                                "data": {
                                    "cmd": check_bytes(command),
                                    "args": check_bytes(b" ".join(args)),
                                },
                                "src_ip": self.transport.getPeer().host,
                                "src_port": self.transport.getPeer().port,
                            }
                        )

                if not (
                    command.lower().startswith(b"user") or command.lower().startswith(b"pass")
                ):
                    self.failResponse("Authentication failed")
                    return None

                if self.blocked is not None:
                    self.blocked.append((command, args))
                    return None

                command = command.upper()
                if not self.mbox and command not in self.AUTH_CMDS:
                    raise POP3Error(b"not authenticated yet: cannot do " + command)
                f = getattr(self, f"do_{check_bytes(command)}", None)
                if f:
                    return f(*args)
                raise POP3Error(b"Unknown protocol command: " + command)

            def do_USER(self, user):  # noqa: N802
                self._user = user
                self.successResponse("USER Ok")

            def do_PASS(self, password: bytes, *words: tuple[bytes]):  # noqa: N802
                if self._user:
                    username = check_bytes(self._user)
                    password = check_bytes(b" ".join((password, *words)))
                    peer = self.transport.getPeer()
                    _q_s.check_login(username, password, ip=peer.host, port=peer.port)
                    self.failResponse("Authentication failed")
                else:
                    self.failResponse("USER first, then PASS")

                self._user = None

        class CustomPOP3Factory(Factory):
            protocol = CustomPOP3Protocol
            portal = None

            def buildProtocol(self, address):  # noqa: N802,ARG002
                p = self.protocol()
                p.portal = self.portal
                p.factory = self
                return p

        factory = CustomPOP3Factory()
        reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from poplib import POP3 as POP3Client  # noqa: N811

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password
            pp = POP3Client(_ip, _port)
            pp.user(_username)
            pp.pass_(_password)


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qpop3server = QPOP3Server(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        qpop3server.run_server()



from __future__ import annotations

from telnetlib import Telnet

import pytest

from honeypots import QTelnetServer
from .utils import (
    assert_connect_is_logged,
    assert_login_is_logged,
    IP,
    load_logs_from_file,
    PASSWORD,
    USERNAME,
    wait_for_server,
)

PORT = "50023"


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QTelnetServer, "port": PORT}],
    indirect=True,
)
def test_telnet_server(server_logs):
    with wait_for_server(PORT):
        telnet_client = Telnet(IP, int(PORT))
        telnet_client.read_until(b"login: ")
        telnet_client.write(USERNAME.encode() + b"\n")
        telnet_client.read_until(b"Password: ")
        telnet_client.write(PASSWORD.encode() + b"\n")

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 2
    connect, login = logs
    assert_connect_is_logged(connect, PORT)
    assert_login_is_logged(login)



from __future__ import annotations

from contextlib import suppress

from twisted.internet import reactor
from twisted.internet.protocol import Factory, Protocol

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    check_bytes,
)


class QRedisServer(BaseServer):
    NAME = "redis_server"
    DEFAULT_PORT = 6379

    def server_main(self):  # noqa: C901
        _q_s = self

        class CustomRedisProtocol(Protocol):
            def get_command(self, data):
                with suppress(Exception):
                    _data = data.decode("utf-8").split("\x0d\x0a")
                    if _data[0][0] == "*":
                        _count = int(_data[0][1]) - 1
                        _data.pop(0)
                        command = self._parse_field(_data, 0)
                        if command:
                            return _count, _data[1::2][0]

                return 0, ""

            def parse_data(self, count: int, data: bytes):
                _data = data.decode("utf-8").split("\r\n")[3::]
                username, password = "", ""
                if count == 2:  # noqa: PLR2004
                    username = self._parse_field(_data, 0)
                    password = self._parse_field(_data, 1)
                elif count == 1:
                    password = self._parse_field(_data, 0)
                if count in {1, 2}:
                    peer = self.transport.getPeer()
                    _q_s.check_login(
                        check_bytes(username), check_bytes(password), ip=peer.host, port=peer.port
                    )

            @staticmethod
            def _parse_field(str_list: list[str], index: int) -> str:
                if str_list[0::2][index][0] == "$" and len(str_list[1::2][index]) == int(
                    str_list[0::2][index][1]
                ):
                    return str_list[1::2][index]
                return ""

            def connectionMade(self):  # noqa: N802
                self._state = 1
                self._variables = {}
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )

            def dataReceived(self, data: bytes):  # noqa: N802
                count, command = self.get_command(data)
                if command == "AUTH":
                    self.parse_data(count, data)
                    self.transport.write(b"-ERR invalid password\r\n")
                else:
                    self.transport.write(f'-ERR unknown command "{command}"\r\n'.encode())
                self.transport.loseConnection()

        factory = Factory()
        factory.protocol = CustomRedisProtocol
        reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from redis import StrictRedis

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password
            r = StrictRedis.from_url(f"redis://{_username}:{_password}@{_ip}:{_port}/1")
            for _ in r.scan_iter("user:*"):
                pass


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qredisserver = QRedisServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        qredisserver.run_server()



from __future__ import annotations

from contextlib import suppress

import pytest
from elasticsearch import Elasticsearch, NotFoundError

from honeypots import QElasticServer
from .utils import (
    assert_connect_is_logged,
    assert_login_is_logged,
    IP,
    load_logs_from_file,
    PASSWORD,
    USERNAME,
    wait_for_server,
)

PORT = "59200"


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QElasticServer, "port": PORT}],
    indirect=True,
)
def test_elastic_server(server_logs):
    with wait_for_server(PORT), suppress(NotFoundError):
        elastic = Elasticsearch(
            [f"https://{IP}:{PORT}"],
            basic_auth=(USERNAME, PASSWORD),
            verify_certs=False,
        )
        elastic.search(index="test", body={}, size=99)

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 3
    connect, login, dump = logs
    assert_connect_is_logged(connect, PORT)
    assert_login_is_logged(login)

    assert "headers" in dump
    assert dump["data"] == "POST /test/_search HTTP/1.1\r\n"



from __future__ import annotations

import json
import logging
import os
import sys
from argparse import ArgumentParser
from collections.abc import Mapping
from contextlib import contextmanager, suppress
from datetime import datetime
from json import JSONEncoder
from logging import DEBUG, Formatter, getLogger, Handler, LogRecord
from logging.handlers import RotatingFileHandler, SysLogHandler
from os import getuid, scandir
from pathlib import Path
from signal import SIGTERM
from socket import AF_INET, SOCK_STREAM, socket
from sqlite3 import connect as sqlite3_connect
from sys import stdout
from tempfile import _get_candidate_names, gettempdir, NamedTemporaryFile
from time import sleep, time
from typing import Any, Iterator, MutableMapping
from urllib.parse import urlparse

import psutil
from OpenSSL import crypto
from psutil import process_iter
#from psycopg2 import connect as psycopg2_connect, sql


def set_up_error_logging():
    _logger = logging.getLogger("honeypots.error")
    if not _logger.handlers:
        _logger.setLevel(logging.INFO)
        handler = logging.StreamHandler(sys.stdout)
        handler.setLevel(logging.INFO)
        formatter = logging.Formatter("[%(levelname)s] %(message)s")
        handler.setFormatter(formatter)
        _logger.addHandler(handler)
    return _logger


logger = set_up_error_logging()


def is_privileged():
    with suppress(Exception):
        return getuid() == 0
    with suppress(Exception):
        import ctypes

        return ctypes.windll.shell32.IsUserAnAdmin() != 0
    return False


def set_local_vars(self, config: str | None = None):
    if not config:
        return
    try:
        config_data = json.loads(Path(config).read_text())
        honeypots = config_data.get("honeypots", [])
        honeypot = self.__class__.__name__[1:-6].lower()
        if honeypot and honeypot in honeypots:
            for attr, value in honeypots[honeypot].items():
                setattr(self, attr, value)
                if attr == "port":
                    self.auto_disabled = True
    except Exception as error:
        logger.debug(f"Setting local variables failed: {error}", exc_info=True)


def _serialize_message(  # noqa: C901
    record: LogRecord,
    custom_filter: dict,
) -> dict | str | None:
    try:
        if custom_filter:
            filters = custom_filter.get("honeypots", {})
            options = filters.get("options", [])
            if "remove_errors" in options and "error" in record.msg:
                return None
            if isinstance(record.msg, MutableMapping):
                if "remove_init" in options and record.msg.get("action") == "process":
                    return None
                if "remove_word_server" in options and "server" in record.msg:
                    record.msg["server"] = record.msg["server"].replace("_server", "")
                for old_key, new_key in filters.get("change", {}).items():
                    if old_key in record.msg:
                        record.msg[new_key] = record.msg.pop(old_key)
                for key in filters.get("remove", []):
                    record.msg.pop(key, None)
                if "contains" in filters and any(k not in record.msg for k in filters["contains"]):
                    return None
        if isinstance(record.msg, Mapping):
            return serialize_object({"timestamp": datetime.utcnow().isoformat(), **record.msg})
        return serialize_object(record.msg)
    except Exception as error:
        return serialize_object({"name": record.name, "error": repr(error)})


def _parse_record(record: LogRecord, custom_filter: dict, type_: str) -> LogRecord | None:
    serialized_msg = _serialize_message(record, custom_filter)
    if not serialized_msg:
        return None
    with suppress(Exception):
        if type_ == "file":
            if custom_filter:
                options = custom_filter.get("honeypots", {}).get("options", [])
                if "dump_json_to_file" in options:
                    record.msg = json.dumps(serialized_msg, sort_keys=True, cls=ComplexEncoder)
        elif type_ == "db_postgres":
            pass
        elif type_ == "db_sqlite":
            for item in ["data", "error"]:
                if item in serialized_msg and not isinstance(serialized_msg[item], str):
                    serialized_msg[item] = repr(serialized_msg[item]).replace("\x00", " ")
        else:
            record.msg = json.dumps(serialized_msg, sort_keys=True, cls=ComplexEncoder)
    return record


def setup_logger(name: str, temp_name: str, config: str | None, drop: bool = False):
    logs = "terminal"
    logs_location = ""
    config_data = {}
    custom_filter = None
    if config:
        try:
            config_data = json.loads(Path(config).read_text())
            logs = config_data.get("logs", logs)
            logs_location = config_data.get("logs_location", logs_location)
            custom_filter = config_data.get("custom_filter", custom_filter)
        except json.JSONDecodeError as error:
            logger.error(f"Could not parse config '{config}' as JSON: {error}")
        except OSError as error:
            logger.error(f"Could not read config file '{config}': {error}")

    logs_path = Path(logs_location) if logs_location else Path(gettempdir()) / "logs"
    logs_path.mkdir(parents=True, exist_ok=True)

    ret_logs_obj = getLogger(temp_name)
    ret_logs_obj.setLevel(DEBUG)
    if "db_postgres_removed" in logs or "db_sqlite" in logs:
        ret_logs_obj.addHandler(CustomHandler(temp_name, logs, custom_filter, config_data, drop))
    elif "terminal" in logs:
        ret_logs_obj.addHandler(CustomHandler(temp_name, logs, custom_filter))
    if "file" in logs:
        server = name[1:].lower().replace("server", "")
        server_config = config_data.get("honeypots", {}).get(server, {})
        file_handler = CustomHandlerFileRotate(
            str(logs_path / server_config.get("log_file_name", temp_name)),
            logs=logs,
            custom_filter=custom_filter,
            maxBytes=server_config.get("max_bytes", 10000),
            backupCount=server_config.get("backup_count", 10),
        )
        ret_logs_obj.addHandler(file_handler)
    if "syslog" in logs:
        syslog_handler = _set_up_syslog_handler(
            config_data.get("syslog_address"),
            config_data.get("syslog_facility"),
        )
        if syslog_handler:
            ret_logs_obj.addHandler(syslog_handler)
    return ret_logs_obj


def _set_up_syslog_handler(address: str | None, facility: int | None) -> Handler | None:
    if not address:
        address = ("localhost", 514)
    else:
        url = urlparse(address)
        if not url.hostname or not url.port:
            logger.error(f"Could not parse syslog address '{address}': host or port not found")
            return None
        address = (url.hostname, url.port)
    handler = SysLogHandler(address=address, facility=facility)
    formatter = Formatter("[%(name)s] [%(levelname)s] - %(message)s")
    handler.setFormatter(formatter)
    return handler


def clean_all():
    for entry in scandir("."):
        if entry.is_file() and entry.name.endswith("_server.py"):
            kill_servers(entry.name)


def kill_servers(name):
    with suppress(Exception):
        for process in process_iter():
            cmdline = " ".join(process.cmdline())
            if "--custom" in cmdline and name in cmdline:
                process.send_signal(SIGTERM)
                process.kill()


def get_free_port():
    port = 0
    with suppress(Exception):
        tcp = socket(AF_INET, SOCK_STREAM)
        tcp.bind(("", 0))
        addr, port = tcp.getsockname()
        tcp.close()
    return port


class ComplexEncoder(JSONEncoder):
    def default(self, obj):
        return repr(obj).replace("\x00", " ")


def serialize_object(obj: Any) -> dict | list | str:
    if isinstance(obj, Mapping):
        return {k: serialize_object(v) for k, v in obj.items()}
    if isinstance(obj, list):
        return [serialize_object(v) for v in obj]
    if isinstance(obj, (int, float)):
        return str(obj)
    if isinstance(obj, bytes):
        obj = obj.decode("utf-8", "ignore")
    elif not isinstance(obj, str):
        obj = repr(obj)
    return obj.replace("\x00", " ")


class CustomHandlerFileRotate(RotatingFileHandler):
    def __init__(self, *args, logs="", custom_filter=None, **kwargs):
        self.logs = logs
        self.custom_filter = custom_filter
        super().__init__(*args, **kwargs)

    def emit(self, record):
        _record = _parse_record(record, self.custom_filter, "file")
        if _record:
            super().emit(_record)


class CustomHandler(Handler):
    def __init__(  # noqa: PLR0913
        self,
        uuid: str = "",
        logs: str = "",
        custom_filter: dict | None = None,
        config: dict | None = None,
        drop: bool = False,
    ):
        self.db = {"db_postgres": None, "db_sqlite": None}
        self.logs = logs
        self.uuid = uuid
        self.custom_filter = custom_filter
        if config and "db_postgres_removed" in self.logs:
            parsed = urlparse(config["postgres"])
            #self.db["db_postgres"] = PostgresClass(
            #    host=parsed.hostname,
            #    port=parsed.port,
            #    username=parsed.username,
            #    password=parsed.password,
            #    db=parsed.path[1:],
            #    uuid=self.uuid,
            #    drop=drop,
            ####
        if config and "db_sqlite" in self.logs:
            self.db["db_sqlite"] = SqliteClass(
                file=config["sqlite_file"], drop=drop, uuid=self.uuid
            )
        super().__init__()

    def emit(self, record: LogRecord):  # noqa: C901,PLR0912
        try:
            if "db_postgres_removed" in self.logs and self.db["db_postgres"]:
                if isinstance(record.msg, list):
                    if record.msg[0] in {"sniffer", "errors"}:
                        self.db["db_postgres"].insert_into_data_safe(
                            record.msg[0],
                            json.dumps(serialize_object(record.msg[1]), cls=ComplexEncoder),
                        )
                elif isinstance(record.msg, Mapping) and "server" in record.msg:
                    self.db["db_postgres"].insert_into_data_safe(
                        "servers",
                        json.dumps(serialize_object(record.msg), cls=ComplexEncoder),
                    )
            if "db_sqlite" in self.logs:
                _record = _parse_record(record, self.custom_filter, "db_sqlite")
                if _record:
                    self.db["db_sqlite"].insert_into_data_safe(_record.msg)
            if "terminal" in self.logs:
                _record = _parse_record(record, self.custom_filter, "terminal")
                if _record:
                    stdout.write(_record.msg + "\n")
            if "syslog" in self.logs:
                _record = _parse_record(record, self.custom_filter, "terminal")
                if _record:
                    stdout.write(_record.msg + "\n")
        except Exception as error:
            if (
                self.custom_filter is not None
                and "honeypots" in self.custom_filter
                and "remove_errors" in self.custom_filter["honeypots"].get("options", [])
            ):
                return
            log_entry = {"error": repr(error), "logger": repr(record)}
            stdout.write(f"{json.dumps(log_entry, sort_keys=True, cls=ComplexEncoder)}\n")
        stdout.flush()

class SqliteClass:
    def __init__(self, file=None, drop=False, uuid=None):
        self.file = file
        self.uuid = uuid
        self.mapped_tables = ["servers"]
        self.servers_table_template = {
            "server": "servers_table",
            "action": None,
            "status": None,
            "src_ip": None,
            "src_port": None,
            "username": None,
            "password": None,
            "dest_ip": None,
            "dest_port": None,
            "data": None,
            "error": None,
        }
        self.wait_until_up()
        if drop:
            self.con = sqlite3_connect(
                self.file, timeout=1, isolation_level=None, check_same_thread=False
            )
            self.cur = self.con.cursor()
            self.drop_db()
            self.drop_tables()
            self.con.close()
        self.con = sqlite3_connect(
            self.file, timeout=1, isolation_level=None, check_same_thread=False
        )
        self.cur = self.con.cursor()
        self.create_tables()

    def wait_until_up(self):
        test = True
        while test:
            with suppress(Exception):
                logger.info(f"{self.uuid} - Waiting on sqlite connection")
                conn = sqlite3_connect(self.file, timeout=1, check_same_thread=False)
                conn.close()
                test = False
            sleep(1)
        logger.info(f"{self.uuid} - sqlite connection is good")

    def drop_db(self):
        with suppress(Exception):
            file = Path(self.file)
            file.unlink(missing_ok=False)

    def drop_tables(self):
        with suppress(Exception):
            for table in self.mapped_tables:
                self.cur.execute(f"DROP TABLE IF EXISTS '{table:s}'")

    def create_tables(self):
        with suppress(Exception):
            self.cur.execute(
                "CREATE TABLE IF NOT EXISTS 'servers_table' (id INTEGER PRIMARY KEY,"
                "date DATETIME DEFAULT CURRENT_TIMESTAMP,server text, action text, "
                "status text, src_ip text, src_port text,dest_ip text, dest_port text, "
                "username text, password text, data text, error text)"
            )

    def insert_into_data_safe(self, obj):
        with suppress(Exception):
            parsed = {k: v for k, v in obj.items() if v is not None}
            dict_ = {**self.servers_table_template, **parsed}
            self.cur.execute(
                "INSERT INTO servers_table ("
                "server, action, status, src_ip, src_port, dest_ip, dest_port, username, "
                "password, data, error) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
                (
                    dict_["server"],
                    dict_["action"],
                    dict_["status"],
                    dict_["src_ip"],
                    dict_["src_port"],
                    dict_["dest_ip"],
                    dict_["dest_port"],
                    dict_["username"],
                    dict_["password"],
                    dict_["data"],
                    dict_["error"],
                ),
            )


def server_arguments():
    _server_parser = ArgumentParser(prog="Server")
    _server_parsergroupdeq = _server_parser.add_argument_group("Initialize Server")
    _server_parsergroupdeq.add_argument(
        "--ip",
        type=str,
        help="Change server ip, current is 0.0.0.0",
        required=False,
        metavar="",
    )
    _server_parsergroupdeq.add_argument(
        "--port", type=int, help="Change port", required=False, metavar=""
    )
    _server_parsergroupdeq.add_argument(
        "--username", type=str, help="Change username", required=False, metavar=""
    )
    _server_parsergroupdeq.add_argument(
        "--password", type=str, help="Change password", required=False, metavar=""
    )
    _server_parsergroupdeq.add_argument(
        "--resolver_addresses",
        type=str,
        help="Change resolver address",
        required=False,
        metavar="",
    )
    _server_parsergroupdeq.add_argument(
        "--domain", type=str, help="A domain to test", required=False, metavar=""
    )
    _server_parsergroupdeq.add_argument(
        "--folders",
        type=str,
        help="folders for smb as name:target,name:target",
        required=False,
        metavar="",
    )
    _server_parsergroupdeq.add_argument(
        "--options", type=str, help="Extra options", metavar="", default=""
    )
    _server_parsergroupdes = _server_parser.add_argument_group("Sinffer options")
    _server_parsergroupdes.add_argument(
        "--filter", type=str, help="setup the Sinffer filter", required=False
    )
    _server_parsergroupdes.add_argument(
        "--interface", type=str, help="sinffer interface E.g eth0", required=False
    )
    _server_parsergroupdef = _server_parser.add_argument_group("Initialize Loging")
    _server_parsergroupdef.add_argument(
        "--config",
        type=str,
        help="config file for logs and database",
        required=False,
        default="",
    )
    _server_parsergroupdea = _server_parser.add_argument_group("Auto Configuration")
    _server_parsergroupdea.add_argument(
        "--docker", action="store_true", help="Run project in docker", required=False
    )
    _server_parsergroupdea.add_argument(
        "--aws", action="store_true", help="Run project in aws", required=False
    )
    _server_parsergroupdea.add_argument(
        "--test", action="store_true", help="Test current server", required=False
    )
    _server_parsergroupdea.add_argument(
        "--custom", action="store_true", help="Run custom server", required=False
    )
    _server_parsergroupdea.add_argument(
        "--auto",
        action="store_true",
        help="Run auto configured with random port",
        required=False,
    )
    _server_parsergroupdef.add_argument("--uuid", type=str, help="unique id", required=False)
    return _server_parser.parse_args()


@contextmanager
def create_certificate() -> Iterator[tuple[str, str]]:
    pk = crypto.PKey()
    pk.generate_key(crypto.TYPE_RSA, 2048)
    certificate = crypto.X509()
    certificate.get_subject().C = "US"
    certificate.get_subject().ST = "OR"
    certificate.get_subject().L = "None"
    certificate.get_subject().O = "None"
    certificate.get_subject().OU = "None"
    certificate.get_subject().CN = next(_get_candidate_names())
    certificate.set_serial_number(0)
    before, after = (0, 60 * 60 * 24 * 365 * 2)
    certificate.gmtime_adj_notBefore(before)
    certificate.gmtime_adj_notAfter(after)
    certificate.set_issuer(certificate.get_subject())
    certificate.set_pubkey(pk)
    certificate.sign(pk, "sha256")
    with NamedTemporaryFile() as cert, NamedTemporaryFile() as key:
        cert_path = Path(cert.name)
        key_path = Path(key.name)
        cert_path.write_bytes(crypto.dump_certificate(crypto.FILETYPE_PEM, certificate))
        key_path.write_bytes(crypto.dump_privatekey(crypto.FILETYPE_PEM, pk))
        yield cert.name, key.name


def check_bytes(string: Any) -> str:
    if isinstance(string, bytes):
        return string.decode("utf-8", errors="replace")
    return str(string)


def load_template(filename: str) -> str:
    file_path = Path(__file__).parent / "data" / filename
    return file_path.read_text()


def get_headers_and_ip_from_request(request, options):
    headers = {}
    client_ip = ""
    with suppress(Exception):
        for item, value in dict(request.requestHeaders.getAllRawHeaders()).items():
            headers.update({check_bytes(item): ",".join(map(check_bytes, value))})
        headers.update({"method": check_bytes(request.method)})
        headers.update({"uri": check_bytes(request.uri)})
    if "fix_get_client_ip" in options:
        with suppress(Exception):
            raw_headers = dict(request.requestHeaders.getAllRawHeaders())
            if b"X-Forwarded-For" in raw_headers:
                client_ip = check_bytes(raw_headers[b"X-Forwarded-For"][0])
            elif b"X-Real-IP" in raw_headers:
                client_ip = check_bytes(raw_headers[b"X-Real-IP"][0])
    if client_ip == "":
        client_ip = request.getClientAddress().host
    return client_ip, headers


def service_has_started(port: int):
    try:
        wait_for_service(port)
        return True
    except TimeoutError:
        return False


def wait_for_service(port: int, interval: float = 0.1, timeout: int = 5.0):
    start_time = time()
    while True:
        if _service_runs(port):
            return
        sleep(interval)
        if time() - start_time > timeout:
            raise TimeoutError()


def _service_runs(port: int) -> bool:
    return any(service.laddr.port == port for service in psutil.net_connections())


@contextmanager
def hide_stderr():
    stderr = sys.stderr
    try:
        with Path(os.devnull).open("w") as devnull:
            sys.stderr = devnull
            yield
    finally:
        sys.stderr = stderr



from __future__ import annotations

from contextlib import suppress
from poplib import error_proto, POP3

import pytest

from honeypots import QPOP3Server
from .utils import (
    assert_connect_is_logged,
    assert_login_is_logged,
    IP,
    load_logs_from_file,
    PASSWORD,
    USERNAME,
    wait_for_server,
)

PORT = "50110"
SERVER_CONFIG = {
    "honeypots": {
        "pop3": {
            "options": ["capture_commands"],
        },
    }
}


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QPOP3Server, "port": PORT, "custom_config": SERVER_CONFIG}],
    indirect=True,
)
def test_pop3_server(server_logs):
    with wait_for_server(PORT), suppress(error_proto):
        client = POP3(IP, int(PORT))
        client.user(USERNAME)
        client.pass_(PASSWORD)

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 4
    connect, cmd1, cmd2, login = logs
    assert_connect_is_logged(connect, PORT)
    assert_login_is_logged(login)

    assert cmd1["action"] == "command"
    assert cmd1["data"] == {"args": USERNAME, "cmd": "USER"}
    assert cmd2["action"] == "command"
    assert cmd2["data"] == {"args": PASSWORD, "cmd": "PASS"}



from contextlib import suppress

from twisted.internet import reactor
from twisted.web.server import Site

from honeypots.base_http_server import BaseHttpServer
from honeypots.helper import (
    server_arguments,
)


class QHTTPServer(BaseHttpServer):
    NAME = "http_server"
    DEFAULT_PORT = 80

    def server_main(self):
        resource = self.MainResource(hp_server=self)
        reactor.listenTCP(self.port, Site(resource))
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from requests import get, post

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password
            get(f"http://{_ip}:{_port}", verify=False)
            post(
                f"http://{_ip}:{_port}/login.html",
                data={"username": (None, _username), "password": (None, _password)},
            )


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qhttpserver = QHTTPServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        qhttpserver.run_server()



from contextlib import suppress
from random import choice

from twisted import cred
from twisted.internet import reactor
from twisted.internet.protocol import Factory
from twisted.mail.imap4 import (
    IMAP4Server,
    IllegalClientResponse,
    IllegalOperation,
    IllegalMailboxEncoding,
)

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    check_bytes,
)


class QIMAPServer(BaseServer):
    NAME = "imap_server"
    DEFAULT_PORT = 143

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.mocking_server = choice(
            [b"OK Microsoft Exchange Server 2003 IMAP4rev1 server version 6.5.6944.0 DC9 ready"]
        )

    def server_main(self):  # noqa: C901
        _q_s = self

        class CustomIMAP4Server(IMAP4Server):
            def parse_command(self, line):
                args = line.split(None, 2)
                rest = None
                if len(args) == 3:  # noqa: PLR2004
                    tag, cmd, rest = args
                elif len(args) == 2:  # noqa: PLR2004
                    tag, cmd = args
                elif len(args) == 1:
                    tag = args[0]
                    self.sendBadResponse(tag, "Missing command")
                    return None
                else:
                    self.sendBadResponse(None, "Null command")
                    return None

                cmd = cmd.upper()

                if "capture_commands" in _q_s.options:
                    _q_s.log(
                        {
                            "action": "command",
                            "data": {
                                "cmd": check_bytes(cmd),
                                "tag": check_bytes(tag),
                                "data": check_bytes(rest),
                            },
                            "src_ip": self.transport.getPeer().host,
                            "src_port": self.transport.getPeer().port,
                        }
                    )

                try:
                    return self.dispatchCommand(tag, cmd, rest)
                except IllegalClientResponse as e:
                    self.sendBadResponse(tag, "Illegal syntax: " + str(e))
                except IllegalOperation as e:
                    self.sendNegativeResponse(tag, "Illegal operation: " + str(e))
                except IllegalMailboxEncoding as e:
                    self.sendNegativeResponse(tag, "Illegal mailbox name: " + str(e))

            def connectionMade(self):  # noqa: N802
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )
                self.sendPositiveResponse(message=_q_s.mocking_server)

            def authenticateLogin(self, user, passwd):  # noqa: N802
                username = check_bytes(user)
                password = check_bytes(passwd)
                peer = self.transport.getPeer()
                _q_s.check_login(username, password, ip=peer.host, port=peer.port)
                raise cred.error.UnauthorizedLogin()

            def lineReceived(self, line: bytes):  # noqa: N802
                with suppress(IndexError):
                    _line = line.split(b" ")[1].lower()
                    if _line.startswith((b"login", b"capability")):
                        super().lineReceived(line)

        class CustomIMAPFactory(Factory):
            protocol = CustomIMAP4Server
            portal = None

            def buildProtocol(self, _):  # noqa: N802
                protocol = self.protocol()
                protocol.portal = self.portal
                protocol.factory = self
                return protocol

        factory = CustomIMAPFactory()
        reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from imaplib import IMAP4

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password
            imap_test = IMAP4(_ip, _port)
            # imap_test.welcome
            imap_test.login(_username, _password)


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qimapserver = QIMAPServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        qimapserver.run_server()



from __future__ import annotations

from contextlib import suppress

import pytest
from redis import AuthenticationError, StrictRedis

from honeypots import QRedisServer
from .utils import (
    assert_connect_is_logged,
    assert_login_is_logged,
    IP,
    load_logs_from_file,
    PASSWORD,
    USERNAME,
    wait_for_server,
)

PORT = "56379"


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QRedisServer, "port": PORT}],
    indirect=True,
)
def test_redis_server(server_logs):
    with wait_for_server(PORT), suppress(AuthenticationError):
        redis = StrictRedis.from_url(f"redis://{USERNAME}:{PASSWORD}@{IP}:{PORT}/1")
        for _ in redis.scan_iter("user:*"):
            pass

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 2
    connect, login = logs
    assert_connect_is_logged(connect, PORT)
    assert_login_is_logged(login)



from contextlib import suppress
from random import randint, uniform
from time import time

from twisted.internet import reactor
from twisted.internet.protocol import Factory, Protocol

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
)


class QMemcacheServer(BaseServer):
    NAME = "memcache_server"
    DEFAULT_PORT = 11211

    def server_main(self):  # noqa: C901
        _q_s = self

        class CustomRedisProtocol(Protocol):
            _state = None

            def get_stats(self) -> bytes:
                items = randint(80000000, 90000000)
                ret = ""
                temp = {
                    "pid": randint(5, 400),
                    "uptime": randint(1000, 2000),
                    "time": int(time()),
                    "version": "1.5.6",
                    "libevent": "2.1.8-stable",
                    "pointer_size": 64,
                    "rusage_user": round(uniform(0.1, 0.9), 4),
                    "rusage_system": round(uniform(0.1, 0.9), 6),
                    "max_connections": 1024,
                    "curr_connections": randint(1, 1024),
                    "total_connections": 5,
                    "rejected_connections": 0,
                    "connection_structures": 2,
                    "reserved_fds": 20,
                    "cmd_get": 0,
                    "cmd_set": 40,
                    "cmd_flush": 0,
                    "cmd_touch": 0,
                    "get_hits": 0,
                    "get_misses": 0,
                    "get_expired": 0,
                    "get_flushed": 0,
                    "delete_misses": 0,
                    "delete_hits": 0,
                    "incr_misses": 0,
                    "incr_hits": 0,
                    "decr_misses": 0,
                    "decr_hits": 0,
                    "cas_misses": 0,
                    "cas_hits": 0,
                    "cas_badval": 0,
                    "touch_hits": 0,
                    "touch_misses": 0,
                    "auth_cmds": 0,
                    "auth_errors": 0,
                    "bytes_read": randint(7000000, 8000000),
                    "bytes_written": randint(500000, 1000000),
                    "limit_maxbytes": 33554432,
                    "accepting_conns": 1,
                    "listen_disabled_num": 0,
                    "time_in_listen_disabled_us": 0,
                    "threads": randint(4, 9000),
                    "conn_yields": 0,
                    "hash_power_level": 16,
                    "hash_bytes": 524288,
                    "hash_is_expanding": False,
                    "slab_reassign_rescues": 0,
                    "slab_reassign_chunk_rescues": 0,
                    "slab_reassign_evictions_nomem": 0,
                    "slab_reassign_inline_reclaim": 0,
                    "slab_reassign_busy_items": 0,
                    "slab_reassign_busy_deletes": 0,
                    "slab_reassign_running": False,
                    "slabs_moved": 0,
                    "lru_crawler_running": 0,
                    "lru_crawler_starts": randint(500000, 700000),
                    "lru_maintainer_juggles": randint(400000, 500000),
                    "malloc_fails": 0,
                    "log_worker_dropped": 0,
                    "log_worker_written": 0,
                    "log_watcher_skipped": 0,
                    "log_watcher_sent": 0,
                    "bytes": randint(13554432, 33554432),
                    "curr_items": items,
                    "total_items": items,
                    "slab_global_page_pool": 0,
                    "expired_unfetched": 0,
                    "evicted_unfetched": 0,
                    "evicted_active": 0,
                    "evictions": 0,
                    "reclaimed": 0,
                    "crawler_reclaimed": 0,
                    "crawler_items_checked": randint(5000, 6000),
                    "lrutail_reflocked": 0,
                    "moves_to_cold": randint(5000, 6000),
                    "moves_to_warm": randint(5000, 6000),
                    "moves_within_lru": 0,
                    "direct_reclaims": 0,
                    "lru_bumps_dropped": 0,
                }

                for key, value in temp.items():
                    ret += f"STAT {key} {value}\r\n"
                ret += "END\r\n"
                return ret.encode()

            def get_key(self, key: bytes) -> bytes:
                try:
                    random = randint(80000000, 90000000)
                    temp = f"VALUE {key.decode()} 0 {len(str(random))}\r\n{random}\r\nEND\r\n"
                    return temp.encode()
                except UnicodeDecodeError:
                    return b""

            def connectionMade(self):  # noqa: N802
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )

            def dataReceived(self, data: bytes):  # noqa: N802
                with suppress(Exception):
                    _data = data.split(b"\r\n")[0].split(b" ")
                    if _data[0] == b"stats":
                        self.transport.write(self.get_stats())
                    elif _data[0] == b"get":
                        self.transport.write(self.get_key(_data[1]))
                    elif _data[0] == b"set" and len(_data) > 4:  # noqa: PLR2004
                        self.transport.write(b"STORED\r\n")
                    else:
                        self.transport.write(b"ERROR\r\n")
                    if _data[0] != b"":
                        _q_s.log(
                            {
                                "action": _data[0].decode(),
                                "src_ip": self.transport.getPeer().host,
                                "src_port": self.transport.getPeer().port,
                            }
                        )
                self.transport.loseConnection()

        factory = Factory()
        factory.protocol = CustomRedisProtocol
        reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):  # noqa: ARG002
        with suppress(Exception):
            from warnings import filterwarnings

            filterwarnings(action="ignore", module=".*socket.*")
            from socket import socket, AF_INET, SOCK_STREAM

            _ip = ip or self.ip
            _port = port or self.port
            c = socket(AF_INET, SOCK_STREAM)
            c.connect((_ip, _port))
            c.send(b"stats\r\n")
            c.recvfrom(10000)
            c.close()


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        QMemcacheServer = QMemcacheServer(
            ip=parsed.ip, port=parsed.port, options=parsed.options, config=parsed.config
        )
        QMemcacheServer.run_server()



from __future__ import annotations

from ftplib import FTP

import pytest

from honeypots import QFTPServer
from .utils import (
    assert_connect_is_logged,
    assert_login_is_logged,
    IP,
    load_logs_from_file,
    PASSWORD,
    USERNAME,
    wait_for_server,
)

PORT = "50021"
SERVER_CONFIG = {
    "honeypots": {
        "ftp": {
            "options": ["capture_commands"],
        },
    }
}


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QFTPServer, "port": PORT, "custom_config": SERVER_CONFIG}],
    indirect=True,
)
def test_ftp_server(server_logs):
    with wait_for_server(PORT):
        client = FTP()
        client.connect(IP, int(PORT))
        client.login(USERNAME, PASSWORD)
        client.pwd()
        client.quit()

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 6
    connect, cmd1, cmd2, login, cmd3, cmd4 = logs
    assert_connect_is_logged(connect, PORT)
    assert_login_is_logged(login)

    assert cmd1["data"] == {"args": f"('{USERNAME}',)", "cmd": "USER"}
    assert cmd2["data"] == {"args": f"('{PASSWORD}',)", "cmd": "PASS"}
    assert cmd3["data"] == {"args": "()", "cmd": "PWD"}
    assert cmd4["data"] == {"args": "()", "cmd": "QUIT"}



from contextlib import suppress
from pathlib import Path
from random import randint
from tempfile import TemporaryDirectory
from threading import current_thread
from unittest.mock import patch

from impacket import smbserver
from impacket.ntlm import compute_lmhash, compute_nthash
from six.moves import socketserver

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    hide_stderr,
)


class QSMBServer(BaseServer):
    NAME = "smb_server"
    DEFAULT_PORT = 445

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.folders = ""

    def server_main(self):  # noqa: C901
        _q_s = self

        class SMBSERVERHandler(smbserver.SMBSERVERHandler):
            def __init__(self, request, client_address, server, select_poll=False):
                self.__SMB = server
                self.__timeOut = 60 * 10
                self.__request = request
                self.__select_poll = select_poll
                self.__ip, self.__port = client_address[:2]
                self.__connId = f"thread_{self.__ip}_{self.__port}_{randint(1000, 9999)}"
                current_thread().name = self.__connId
                socketserver.BaseRequestHandler.__init__(self, request, client_address, server)

        class SMBServer(smbserver.SMBSERVER):
            def __init__(self, server_address, handler_class=SMBSERVERHandler, config_parser=None):
                super().__init__(server_address, handler_class, config_parser)

            def processRequest(self, connId, data):  # noqa: N802,N803
                # hide trace logging from smbserver module
                with hide_stderr():
                    return super().processRequest(connId, data)

            def log(self, msg, level=None):  # noqa: ARG002
                temp = current_thread().name
                if not temp.startswith("thread_") or temp.count("_") < 2:  # noqa: PLR2004
                    return
                _, ip, port, *_ = temp.split("_")
                message = msg.strip()
                if (
                    "Incoming connection" in message
                    or "AUTHENTICATE_MESSAGE" in message
                    or "authenticated successfully" in message
                ):
                    _q_s.log(
                        {
                            "action": "connection",
                            "data": message,
                            "src_ip": ip,
                            "src_port": port,
                        }
                    )
                elif ":aaaaaaaaaaaaaaaa:" in message:
                    with suppress(ValueError):
                        username, _, _, _, nt_res_1, nt_res_2 = message.split(":")
                        _q_s.log(
                            {
                                "action": "login",
                                "username": username,
                                "src_ip": ip,
                                "src_port": port,
                                "data": {"nt_data_1": nt_res_1, "nt_data_2": nt_res_2},
                            }
                        )

        class SimpleSMBServer(smbserver.SimpleSMBServer):
            def __init__(self, listenAddress="0.0.0.0", listenPort=445, configFile=""):  # noqa: N803
                with patch("impacket.smbserver.SMBSERVER", SMBServer):
                    super().__init__(listenAddress, listenPort, configFile)

            def start(self):
                self.__srvsServer.start()
                self.__wkstServer.start()
                self.__server.serve_forever()

        with TemporaryDirectory() as tmpdir:
            server = SimpleSMBServer(listenAddress=self.ip, listenPort=self.port)
            if self.folders == "" or self.folders is None:
                server.addShare("C$", tmpdir, "", readOnly="yes")
            else:
                for folder in self.folders.split(","):
                    name, path = folder.split(":")
                    if Path(path).is_dir() and len(name) > 0:
                        server.addShare(name, path, "", readOnly="yes")

            server.setSMB2Support(True)
            server.addCredential(
                self.username, 0, compute_lmhash(self.password), compute_nthash(self.password)
            )
            server.setSMBChallenge("")
            server.start()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from impacket.smbconnection import SMBConnection

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password
            smb_client = SMBConnection(_ip, _ip, sess_port=_port)
            smb_client.login(_username, _password)


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qsmbserver = QSMBServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            folders=parsed.folders,
            options=parsed.options,
            config=parsed.config,
        )
        qsmbserver.run_server()



from contextlib import suppress

from twisted.internet import reactor
from twisted.protocols.sip import Base

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    check_bytes,
)


class QSIPServer(BaseServer):
    NAME = "sip_server"
    DEFAULT_PORT = 5060

    def server_main(self):
        _q_s = self

        class CustomSIPServer(Base):
            def handle_request(self, message, addr):
                headers = {}

                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": addr[0],
                        "src_port": addr[1],
                    }
                )

                for item, value in message.headers.items():
                    headers.update({check_bytes(item): ",".join(map(check_bytes, value))})

                _q_s.log(
                    {
                        "action": "request",
                        "src_ip": addr[0],
                        "src_port": addr[1],
                        "data": headers,
                    }
                )
                response = self.responseFromRequest(200, message)
                response.creationFinished()
                self.deliverResponse(response)

        reactor.listenUDP(port=self.port, protocol=CustomSIPServer(), interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from socket import socket, AF_INET, SOCK_DGRAM, IPPROTO_UDP

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password
            sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)
            sock.sendto(
                b"INVITE sip:user_1@test.test SIP/2.0\r\n"
                b"To: <sip:user_2@test.test>\r\n"
                b"From: sip:user_3@test.test.test;tag=none\r\n"
                b"Call-ID: 1@0.0.0.0\r\n"
                b"CSeq: 1 INVITE\r\n"
                b"Contact: sip:user_3@test.test.test\r\n"
                b"Via: SIP/2.0/TCP 0.0.0.0;branch=34uiddhjczqw3mq23\r\n"
                b"Content-Length: 1\r\n\r\nT",
                (_ip, _port),
            )
            sock.close()


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        QSIPServer = QSIPServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        QSIPServer.run_server()



from .dhcp_server import QDHCPServer
from .dns_server import QDNSServer
from .elastic_server import QElasticServer
from .ftp_server import QFTPServer
from .helper import (
    is_privileged,
    clean_all,
    get_free_port,
    kill_servers,
    server_arguments,
    set_local_vars,
    setup_logger,
    set_up_error_logging,
)
from .http_proxy_server import QHTTPProxyServer
from .http_server import QHTTPServer
from .https_server import QHTTPSServer
from .imap_server import QIMAPServer
from .ipp_server import QIPPServer
from .irc_server import QIRCServer
from .ldap_server import QLDAPServer
from .memcache_server import QMemcacheServer
from .mssql_server import QMSSQLServer
from .mysql_server import QMysqlServer
from .ntp_server import QNTPServer
from .oracle_server import QOracleServer
from .pjl_server import QPJLServer
from .pop3_server import QPOP3Server
from .postgres_server import QPostgresServer
from .sniffer import QSniffer
from .rdp_server import QRDPServer
from .redis_server import QRedisServer
from .sip_server import QSIPServer
from .smb_server import QSMBServer
from .smtp_server import QSMTPServer
from .snmp_server import QSNMPServer
from .socks5_server import QSOCKS5Server
from .ssh_server import QSSHServer
from .telnet_server import QTelnetServer
from .vnc_server import QVNCServer

__all__ = [
    "QSniffer",
    "QDHCPServer",
    "QDNSServer",
    "QElasticServer",
    "QFTPServer",
    "QHTTPProxyServer",
    "QHTTPSServer",
    "QHTTPServer",
    "QIMAPServer",
    "QIPPServer",
    "QIRCServer",
    "QLDAPServer",
    "QMSSQLServer",
    "QMemcacheServer",
    "QMysqlServer",
    "QNTPServer",
    "QOracleServer",
    "QPJLServer",
    "QPOP3Server",
    "QPostgresServer",
    "QRDPServer",
    "QRedisServer",
    "QSIPServer",
    "QSMBServer",
    "QSMTPServer",
    "QSNMPServer",
    "QSOCKS5Server",
    "QSSHServer",
    "QTelnetServer",
    "QVNCServer",
    "is_privileged",
    "clean_all",
    "get_free_port",
    "kill_servers",
    "server_arguments",
    "set_local_vars",
    "setup_logger",
    "set_up_error_logging",
]



from __future__ import annotations

from contextlib import suppress

import pymssql
import pytest

from honeypots import QMSSQLServer
from .utils import (
    assert_connect_is_logged,
    assert_login_is_logged,
    IP,
    load_logs_from_file,
    PASSWORD,
    USERNAME,
    wait_for_server,
)

PORT = "51433"


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QMSSQLServer, "port": PORT}],
    indirect=True,
)
def test_mssql_server(server_logs):
    with wait_for_server(PORT), suppress(pymssql.OperationalError):
        connection = pymssql.connect(
            host=IP,
            port=str(PORT),
            user=USERNAME,
            password=PASSWORD,
            database="dbname",
        )
        connection.close()

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 2
    connect, login = logs
    assert_connect_is_logged(connect, PORT)
    assert_login_is_logged(login)



from base64 import b64encode, b64decode
from contextlib import suppress
from http.server import SimpleHTTPRequestHandler, ThreadingHTTPServer
from json import dumps
from ssl import SSLContext, PROTOCOL_TLS_SERVER
from urllib.parse import urlparse
from zlib import compressobj, DEFLATED

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    create_certificate,
    check_bytes,
)


class QElasticServer(BaseServer):
    NAME = "elastic_server"
    DEFAULT_PORT = 9200
    DEFAULT_USERNAME = "elastic"

    def server_main(self):  # noqa: C901
        _q_s = self

        class CustomElasticServerHandler(SimpleHTTPRequestHandler):
            server_version = ""
            sys_version = ""

            def _dump_headers(self):
                headers = {}
                with suppress(Exception):
                    for item, value in dict(self.headers).items():
                        headers.update({check_bytes(item): check_bytes(value)})

                _q_s.log(
                    {
                        "action": "dump",
                        "data": check_bytes(self.raw_requestline),
                        "src_ip": self.client_address[0],
                        "src_port": self.client_address[1],
                        "headers": headers,
                    }
                )
                return headers

            def _remove_headers(self, headers):
                for header in headers:
                    for current_header in self._headers_buffer[:]:
                        if header.lower() in current_header.lower():
                            self._headers_buffer.remove(current_header)

            def _compress_gzip(self, content):
                compressobj_z = compressobj(-1, DEFLATED, 31)
                return compressobj_z.compress(content) + compressobj_z.flush()

            def _set_response_gzip(self, content, code):
                self.send_response(code)
                gzip_compressed_data = self._compress_gzip(content)
                self.send_header("content-encoding", "gzip")
                self.send_header("content-length", str(len(gzip_compressed_data)))
                self.send_header("content-type", "application/json; charset=UTF-8")
                self.end_headers()
                return gzip_compressed_data

            def do_HEAD(self):  # noqa: N802
                self.send_response(200)
                self.send_header("content-encoding", "gzip")
                self.send_header("content-type", "application/json; charset=UTF-8")
                self.end_headers()

            def _set_response_gzip_auth(self, content, code):
                self._dump_headers()
                self.send_response(code)
                self._remove_headers([b"server:", b"date:"])
                gzip_compressed_data = self._compress_gzip(content)
                self.send_header("content-encoding", "gzip")
                self.send_header("content-length", str(len(gzip_compressed_data)))
                self.send_header("content-type", "application/json; charset=UTF-8")
                self.send_header("WWW-Authenticate", 'Basic realm="security" charset="UTF-8"')
                self.end_headers()
                return gzip_compressed_data

            def do_GET(self):  # noqa: N802
                username = ""
                password = ""
                e_name = "045dffec8b60"
                e_cluster_name = "R&DBackup"
                e_host = "172.17.0.2"
                e_transport_address = e_host + ":9300"
                e_build_type = "en"
                e_os_name = "Linux"
                e_os_pretty_name = "CentOS Linux 8"
                e_os_version = "5.8.0-53-generic"

                key = self.server.get_auth_key()
                if self.headers.get("Authorization") is None:
                    _q_s.log(
                        {
                            "action": "login",
                            "status": "failed",
                            "src_ip": self.client_address[0],
                            "src_port": self.client_address[1],
                            "username": username,
                            "password": password,
                        }
                    )
                    error = {
                        "type": "security_exception",
                        "reason": f"unable to authenticate user [{username}] for REST request [/]",
                        "header": {"WWW-Authenticate": 'Basic realm="security" charset="UTF-8"'},
                    }
                    self._send_auth_error(error, 401)
                elif self.headers.get("Authorization") == "Basic " + str(key):
                    _q_s.log(
                        {
                            "action": "login",
                            "status": "success",
                            "src_ip": self.client_address[0],
                            "src_port": self.client_address[1],
                            "username": _q_s.username,
                            "password": _q_s.password,
                        }
                    )
                    with suppress(Exception):
                        extracted = urlparse(self.path).path
                    if extracted == "/":
                        normal_payload = {
                            "name": e_name,
                            "cluster_name": e_cluster_name,
                            "cluster_uuid": "09cf5BKcTCG2U8z2ndwGEw",
                            "version": {
                                "number": "7.12.1",
                                "build_flavor": "default",
                                "build_type": e_build_type,
                                "build_hash": "3186837139b9c6b6d23c3200870651f10d3343b7",
                                "build_date": "2021-04-20T20:56:39.040728659Z",
                                "build_snapshot": False,
                                "lucene_version": "8.8.0",
                                "minimum_wire_compatibility_version": "6.8.0",
                                "minimum_index_compatibility_version": "6.0.0-beta1",
                            },
                            "tagline": "You Know, for Search",
                        }
                        self.wfile.write(
                            self._set_response_gzip(dumps(normal_payload).encode("utf-8"), 200)
                        )
                    elif extracted.startswith("/_nodes"):
                        _nodes_payload = {
                            "_nodes": {"total": 1, "successful": 1, "failed": 0},
                            "cluster_name": e_cluster_name,
                            "nodes": {
                                "rvyTV3xvTgyt74ti4u12bw": {
                                    "name": e_name,
                                    "transport_address": e_transport_address,
                                    "host": e_host,
                                    "src_ip": e_host,
                                    "version": "7.12.1",
                                    "build_flavor": "default",
                                    "build_type": e_build_type,
                                    "build_hash": "3186837139b9c6b6d23c3200870651f10d3343b7",
                                    "roles": [
                                        "data",
                                        "data_cold",
                                        "data_content",
                                        "data_frozen",
                                        "data_hot",
                                        "data_warm",
                                        "ingest",
                                        "master",
                                        "ml",
                                        "remote_cluster_client",
                                        "transform",
                                    ],
                                    "attributes": {
                                        "ml.machine_memory": "16685318144",
                                        "xpack.installed": "true",
                                        "transform.node": "true",
                                        "ml.max_open_jobs": "20",
                                        "ml.max_jvm_size": "8342470656",
                                    },
                                    "process": {
                                        "refresh_interval_in_millis": 1000,
                                        "id": 7,
                                        "mlockall": False,
                                    },
                                },
                                "os": {
                                    "refresh_interval_in_millis": 1000,
                                    "name": e_os_name,
                                    "pretty_name": e_os_pretty_name,
                                    "arch": "amd64",
                                    "version": e_os_version,
                                    "available_processors": 32,
                                    "allocated_processors": 8,
                                },
                                "process": {
                                    "refresh_interval_in_millis": 1000,
                                    "id": 7,
                                    "mlockall": False,
                                },
                            },
                        }
                        self.wfile.write(
                            self._set_response_gzip(dumps(_nodes_payload).encode("utf-8"), 200)
                        )
                    elif extracted.startswith("/_cluster/health"):
                        _cluster_health_payload = {
                            "cluster_name": e_cluster_name,
                            "status": "green",
                            "timed_out": False,
                            "number_of_nodes": 1,
                            "number_of_data_nodes": 1,
                            "active_primary_shards": 0,
                            "active_shards": 0,
                            "relocating_shards": 0,
                            "initializing_shards": 0,
                            "unassigned_shards": 0,
                            "delayed_unassigned_shards": 0,
                            "number_of_pending_tasks": 0,
                            "number_of_in_flight_fetch": 0,
                            "task_max_waiting_in_queue_millis": 0,
                            "active_shards_percent_as_number": 100.0,
                        }
                        self._send_auth_payload(_cluster_health_payload, 200)
                    elif extracted.startswith("/_"):
                        _index = extracted.split("/")[1].lower()
                        error = {
                            "type": "invalid_index_name_exception",
                            "reason": f'Invalid index name [{_index}], must not start with "_".',
                            "index_uuid": "_na_",
                            "index": _index,
                        }
                        self._send_auth_error(error, 400)
                    else:
                        _search = extracted.split("/")[1].lower()
                        error = {
                            "type": "index_not_found_exception",
                            "reason": f"no such index [{_search}]",
                            "resource.type": "index_or_alias",
                            "resource.id": _search,
                            "index_uuid": "_na_",
                            "index": _search,
                        }
                        self._send_auth_error(error, 404)
                else:
                    authorization_string = self.headers.get("Authorization").split(" ")
                    basic = b64decode(authorization_string[1]).decode("utf-8")
                    username, password = basic.split(":")
                    _q_s.log(
                        {
                            "action": "login",
                            "status": "failed",
                            "src_ip": self.client_address[0],
                            "src_port": self.client_address[1],
                            "username": username,
                            "password": password,
                        }
                    )
                    error = {
                        "type": "security_exception",
                        "reason": "missing authentication credentials for REST request [/]",
                        "header": {"WWW-Authenticate": 'Basic realm="security" charset="UTF-8"'},
                    }
                    self._send_auth_error(error, 401)

            def _send_auth_error(self, error: dict, status_code: int):
                auth_payload = {
                    "error": {"root_cause": [error], **error},
                    "status": status_code,
                }
                self._send_auth_payload(auth_payload, status_code)

            def _send_auth_payload(self, payload: dict, status_code: int):
                compressed = self._set_response_gzip_auth(
                    dumps(payload).encode("utf-8"), status_code
                )
                self.wfile.write(compressed)

            do_POST = do_GET  # noqa: N815
            do_PUT = do_GET  # noqa: N815
            do_DELETE = do_GET  # noqa: N815

            def send_error(self, code, message=None, explain=None):
                self.error_message_format = "Error!"
                super().send_error(code, message, explain)

            def log_message(self, *_, **__):
                pass

            def handle_one_request(self):
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.client_address[0],
                        "src_port": self.client_address[1],
                    }
                )
                return super().handle_one_request()

        class CustomElasticServer(ThreadingHTTPServer):
            key = b64encode(b"elastic:changeme").decode("ascii")

            def __init__(self, address, handler_class=CustomElasticServerHandler):
                super().__init__(address, handler_class)

            def set_auth_key(self, username, password):
                self.key = b64encode(f"{username}:{password}".encode()).decode("ascii")

            def get_auth_key(self):
                return self.key

        with create_certificate() as (cert, key):
            server = CustomElasticServer((self.ip, self.port))
            server.set_auth_key(self.username, self.password)
            ctx = SSLContext(PROTOCOL_TLS_SERVER)
            ctx.load_cert_chain(certfile=cert, keyfile=key)
            server.socket = ctx.wrap_socket(
                server.socket,
                server_side=True,
            )
            server.serve_forever()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from elasticsearch import Elasticsearch

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password
            es = Elasticsearch(
                [f"https://{_ip}:{_port}"],
                http_auth=(_username, _password),
                verify_certs=False,
            )
            es.search(index="test", body={}, size=99)


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qelasticserver = QElasticServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        qelasticserver.run_server()



from __future__ import annotations

import re
from binascii import hexlify
from multiprocessing import Process
from sys import stdout
from typing import Iterable, TYPE_CHECKING

from netifaces import AF_INET, AF_LINK, ifaddresses
from scapy.layers.inet import IP, TCP
from scapy.sendrecv import send, sniff

from honeypots.base_server import BaseServer
from honeypots.helper import server_arguments

ICMP_CODES = [
    (0, 0, "Echo/Ping reply"),
    (3, 0, "Destination network unreachable"),
    (3, 1, "Destination host unreachable"),
    (3, 2, "Destination protocol unreachable"),
    (3, 3, "Destination port unreachable"),
    (3, 4, "Fragmentation required"),
    (3, 5, "Source route failed"),
    (3, 6, "Destination network unknown"),
    (3, 7, "Destination host unknown"),
    (3, 8, "Source host isolated"),
    (3, 9, "Network administratively prohibited"),
    (3, 10, "Host administratively prohibited"),
    (3, 11, "Network unreachable for TOS"),
    (3, 12, "Host unreachable for TOS"),
    (3, 13, "Communication administratively prohibited"),
    (3, 14, "Host Precedence Violation"),
    (3, 15, "Precedence cutoff in effect"),
    (4, 0, "Source quench"),
    (5, 0, "Redirect Datagram for the Network"),
    (5, 1, "Redirect Datagram for the Host"),
    (5, 2, "Redirect Datagram for the TOS & network"),
    (5, 3, "Redirect Datagram for the TOS & host"),
    (8, 0, "Echo/Ping Request"),
    (9, 0, "Router advertisement"),
    (10, 0, "Router discovery/selection/solicitation"),
    (11, 0, "TTL expired in transit"),
    (11, 1, "Fragment reassembly time exceeded"),
    (12, 0, "Pointer indicates the error"),
    (12, 1, "Missing a required option"),
    (12, 2, "Bad length"),
    (13, 0, "Timestamp"),
    (14, 0, "Timestamp Reply"),
    (15, 0, "Information Request"),
    (16, 0, "Information Reply"),
    (17, 0, "Address Mask Request"),
    (18, 0, "Address Mask Reply"),
    (30, 0, "Information Request"),
]
TCP_SYN_FLAG = 0b10

if TYPE_CHECKING:
    from scapy.packet import Packet


class QSniffer(BaseServer):
    NAME = "sniffer"

    def __init__(self, filter_=None, interface=None, config="", **kwargs):
        super().__init__(config=config, **kwargs)
        self.current_ip = ifaddresses(interface)[AF_INET][0]["addr"].encode("utf-8")
        self.current_mac = ifaddresses(interface)[AF_LINK][0]["addr"].encode("utf-8")
        self.filter = filter_
        self.interface = interface
        self.method = "TCPUDP"
        self.allowed_ports = []
        self.allowed_ips = []
        self.common = re.compile(rb"pass|user|login")

    @staticmethod
    def find_icmp(type_, code):
        for icmp_type, icmp_code, msg_type in ICMP_CODES:
            if type_ == icmp_type and code == icmp_code:
                return msg_type
        return "None"

    @staticmethod
    def get_layers(packet: Packet) -> Iterable[str]:
        try:
            yield packet.name
            while packet.payload:
                packet = packet.payload
                yield packet.name
        except AttributeError:
            pass

    def server_main(self):
        try:
            sniff(filter=self.filter, iface=self.interface, prn=self.capture_logic)
        except PermissionError as error:
            self.logger.error(f"Could not start sniffer: {error}")

    def _get_payloads(self, layers: list[str], packet: Packet):
        hex_payloads, raw_payloads, _fields = {}, {}, {}
        for layer in layers:
            try:
                _fields[layer] = packet[layer].fields
                if "load" in _fields[layer]:
                    raw_payloads[layer] = _fields[layer]["load"]
                    hex_payloads[layer] = hexlify(_fields[layer]["load"])
                    if re.search(self.common, raw_payloads[layer]):
                        self.log(
                            {
                                "action": "creds_check",
                                "payload": raw_payloads[layer],
                            },
                        )
            except Exception as error:
                self._log_error(error, 1)
        return _fields, hex_payloads, raw_payloads

    def capture_logic(self, packet: Packet):
        _layers: list[str] = list(self.get_layers(packet))
        _fields, hex_payloads, raw_payloads = self._get_payloads(_layers, packet)

        try:
            if self.method == "ALL":
                try:
                    self.log(
                        {
                            "action": "all",
                            "layers": _layers,
                            "fields": _fields,
                            "payload": hex_payloads,
                        },
                    )
                except Exception as error:
                    self._log_error(error, 2)
            elif (
                self.method == "TCPUDP"
                and packet.haslayer("IP")
                and len(hex_payloads) > 0
                and packet["IP"].src != self.current_ip
            ):
                self._log_tcp_udp(packet, hex_payloads, raw_payloads)

            if (
                packet.haslayer("IP")
                and packet.haslayer("ICMP")
                and packet["IP"].src != self.current_ip
            ):
                self.log(
                    {
                        "action": "icmp",
                        "dest_ip": packet["IP"].src,
                        "dst_ip": packet["IP"].dst,
                        "ICMP_Code": packet["ICMP"].code,
                        "ICMP_Type": packet["ICMP"].type,
                        "ICMP_MSG": self.find_icmp(packet["ICMP"].type, packet["ICMP"].code),
                    },
                )

            if (
                packet.haslayer("IP")
                and packet.haslayer("TCP")
                and packet["IP"].src != self.current_ip
                and packet["TCP"].flags == TCP_SYN_FLAG
            ):
                self._handle_tcp_scan(packet, hex_payloads, raw_payloads)

        except Exception as error:
            self._log_error(error, 5)

        stdout.flush()

    def _handle_tcp_scan(self, packet: Packet, hex_payloads: dict, raw_payloads: dict):
        self.log(
            {
                "action": "tcpscan",
                "dest_ip": packet["IP"].src,
                "dest_port": packet["TCP"].sport,
                "dst_ip": packet["IP"].dst,
                "dst_port": packet["TCP"].dport,
                "raw_payload": raw_payloads,
                "payload": hex_payloads,
            },
        )
        ip_pkg = IP(dst=packet["IP"].src, src=packet["IP"].dst)
        tcp_pkg = TCP(
            dport=packet["TCP"].sport,
            sport=packet["TCP"].dport,
            ack=(packet["TCP"].seq + 1),
            flags="SA",
        )
        send(ip_pkg / tcp_pkg, verbose=False)

    def _log_tcp_udp(self, packet: Packet, hex_payloads: dict, raw_payloads: dict):
        for layer in ["TCP", "UDP"]:
            if packet.haslayer(layer):
                try:
                    self.log(
                        {
                            "action": f"{layer.lower()}payload",
                            "dest_ip": packet["IP"].src,
                            "dest_port": packet[layer].sport,
                            "dst_ip": packet["IP"].dst,
                            "dst_port": packet[layer].dport,
                            "raw_payload": raw_payloads,
                            "payload": hex_payloads,
                        },
                    )
                except Exception as error:
                    self._log_error(error, 3)

    def log(self, log_data: dict):
        log_data.update({"ip": self.current_ip, "mac": self.current_mac})
        self.logs.info([self.NAME, log_data])

    def _log_error(self, error: Exception, _id: int):
        self.logs.error(
            [
                "errors",
                {"error": f"capture_logic_{_id}", "type": f"error -> {error!r}"},
            ]
        )

    def run_sniffer(self, process=None):
        if process:
            self._server_process = Process(name="QSniffer_", target=self.server_main)
            self._server_process.start()
        else:
            self.server_main()


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qsniffer = QSniffer(
            filter_=parsed.filter, interface=parsed.interface, config=parsed.config
        )
        qsniffer.run_sniffer()



import struct
from contextlib import suppress
from socketserver import TCPServer, StreamRequestHandler, ThreadingMixIn
from struct import unpack

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    check_bytes,
)

USER_PW_AUTH_V1 = 1
SOCKS_V5 = 5
AUTH_TYPE_USER_PW = 2


class QSOCKS5Server(BaseServer):
    NAME = "socks5_server"
    DEFAULT_PORT = 1080

    def server_main(self):
        _q_s = self

        class CustomStreamRequestHandler(StreamRequestHandler):
            def handle(self):
                src_ip, src_port = self.client_address
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": src_ip,
                        "src_port": src_port,
                    }
                )
                try:
                    # see RFC 1928
                    version, auth_types_len = unpack("!BB", self.connection.recv(2))
                    if version == SOCKS_V5:
                        supported_auth_methods = unpack(
                            "!" + "B" * auth_types_len, self.connection.recv(auth_types_len)
                        )
                        if AUTH_TYPE_USER_PW in supported_auth_methods:
                            self.connection.sendall(b"\x05\x02")
                            self._check_user_pw_auth(src_ip, src_port)
                except ConnectionResetError:
                    _q_s.logger.debug(
                        f"[{_q_s.NAME}]: Connection reset error when trying to handle connection"
                    )
                except struct.error:
                    _q_s.logger.debug(f"[{_q_s.NAME}]: Could not parse data to handle connection")

                self.server.close_request(self.request)

            def _check_user_pw_auth(self, ip: str, port: int):
                # see RFC 1929
                auth_version = unpack("B", self.connection.recv(1))[0]
                if auth_version == USER_PW_AUTH_V1:
                    _len = ord(self.connection.recv(1))
                    username = check_bytes(self.connection.recv(_len))
                    _len = ord(self.connection.recv(1))
                    password = check_bytes(self.connection.recv(_len))
                    _q_s.check_login(username, password, ip, port)

        class ThreadingTCPServer(ThreadingMixIn, TCPServer):
            pass

        TCPServer.allow_reuse_address = True
        server = ThreadingTCPServer((self.ip, self.port), CustomStreamRequestHandler)
        server.serve_forever()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from requests import get

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password
            get(
                "https://yahoo.com",
                proxies={
                    "http": f"socks5://{_username}:{_password}@{_ip}:{_port}",
                    "https": f"socks5://{_username}:{_password}@{_ip}:{_port}",
                },
            )


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        QSOCKS5Server = QSOCKS5Server(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        QSOCKS5Server.run_server()



from binascii import hexlify, unhexlify
from contextlib import suppress
from struct import pack, unpack

from twisted.internet import reactor
from twisted.internet.protocol import Factory, Protocol

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    check_bytes,
)


class PacketType:
    login = 0x10
    pre_login = 0x12


class QMSSQLServer(BaseServer):
    NAME = "mssql_server"
    DEFAULT_PORT = 1433

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.file_name = None

    def server_main(self):  # noqa: C901
        _q_s = self

        class CustomMSSQLProtocol(Protocol):
            _state = None

            def create_payload(
                self, server_name: str = "", token_error_msg: str = "", error_code=2
            ) -> bytes:
                with suppress(Exception):
                    if server_name == "":
                        server_name = "R&Dbackup"
                    if token_error_msg == "":
                        token_error_msg = (
                            "An error has occurred while establishing a connection to the server."
                        )
                    server_name_hex_len = hexlify(pack("b", len(server_name)))
                    token_error_msg_hex_len = hexlify(pack("<H", len(token_error_msg)))
                    error_code_hex = hexlify(pack("<I", error_code))
                    token_error_hex = (
                        error_code_hex
                        + b"010e"
                        + token_error_msg_hex_len
                        + hexlify(token_error_msg.encode("utf-16-le"))
                        + server_name_hex_len
                        + hexlify(server_name.encode("utf-16-le"))
                        + b"0001000000"
                    )
                    token_done_hex = b"fd020000000000000000000000"
                    token_error_len = hexlify(pack("<H", len(unhexlify(token_error_hex))))
                    data_stream = (
                        b"0401007600350100aa" + token_error_len + token_error_hex + token_done_hex
                    )
                    data_len = hexlify(pack(">H", len(unhexlify(data_stream))))
                    return unhexlify(data_stream[0:4] + data_len + data_stream[8:])

            def connectionMade(self):  # noqa: N802
                self._state = 1
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )

            def dataReceived(self, data: bytes):  # noqa: N802
                if self._state == 1:
                    version = "11000000"
                    packet_type = data[0]
                    if packet_type == PacketType.pre_login:
                        self.transport.write(
                            bytes.fromhex(
                                "0401002500000100000015000601001b000102"
                                f"001c000103001d0000ff{version}00000200"
                            )
                        )
                    elif packet_type == PacketType.login:
                        self._handle_login(data)
                else:
                    self.transport.loseConnection()

            def _handle_login(self, data: bytes):
                value_start, value_length = unpack("=HH", data[48:52])
                username = (
                    data[8 + value_start : 8 + value_start + (value_length * 2)]
                    .replace(b"\x00", b"")
                    .decode("utf-8")
                )
                value_start, value_length = unpack("=HH", data[52:56])
                password = data[8 + value_start : 8 + value_start + (value_length * 2)]
                password = password.replace(b"\x00", b"").replace(b"\xa5", b"")
                password_decrypted = ""
                for x in password:
                    password_decrypted += chr(((x ^ 0xA5) & 0x0F) << 4 | ((x ^ 0xA5) & 0xF0) >> 4)
                peer = self.transport.getPeer()
                _q_s.check_login(
                    check_bytes(username), password_decrypted, ip=peer.host, port=peer.port
                )
                payload = self.create_payload(token_error_msg="Login Failed", error_code=18456)
                self.transport.write(payload)

            def connectionLost(self, reason):  # noqa: N802,ARG002
                self._state = None

        factory = Factory()
        factory.protocol = CustomMSSQLProtocol
        reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from pymssql import connect as pconnect

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password
            conn = pconnect(
                host=_ip, port=str(_port), user=_username, password=_password, database="dbname"
            )
            conn.cursor()


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        QMSSQLServer = QMSSQLServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        QMSSQLServer.run_server()



from __future__ import annotations

import pytest

from honeypots import QIRCServer
from .utils import (
    assert_connect_is_logged,
    connect_to,
    IP,
    load_logs_from_file,
    PASSWORD,
    wait_for_server,
)

PORT = "56667"
SERVER_CONFIG = {
    "honeypots": {
        "irc": {
            "options": ["capture_commands"],
        },
    }
}


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QIRCServer, "port": PORT, "custom_config": SERVER_CONFIG}],
    indirect=True,
)
def test_irc_server(server_logs):
    with wait_for_server(PORT), connect_to(IP, PORT) as connection:
        connection.setblocking(False)
        connection.send(f"PASS {PASSWORD}\n".encode())

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 2
    connect, command = logs
    assert_connect_is_logged(connect, PORT)

    assert command["action"] == "command"
    assert command["data"] == {"command": "PASS", "params": f"['{PASSWORD}']", "prefix": ""}



from __future__ import annotations

from contextlib import suppress
import struct

from twisted.internet import reactor
from twisted.web.resource import Resource
from twisted.web.server import Site

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    check_bytes,
    get_headers_and_ip_from_request,
)

SUPPORTED_OPERATIONS = {
    0x0001: "Reserved",
    0x0002: "Print-Job",
    0x0003: "Print-URI",
    0x0004: "Validate-Job",
    0x0005: "Create-Job",
    0x0006: "Send-Document",
    0x0007: "Send-URI",
    0x0008: "Cancel-Job",
    0x0009: "Get-Job-Attributes",
    0x000A: "Get-Jobs",
    0x000B: "Get-Printer-Attributes",
    0x000C: "Hold-Job",
    0x000D: "Release-Job",
    0x000E: "Restart-Job",
    0x000F: "Reserved",
    0x0010: "Pause-Printer",
    0x0011: "Resume-Printer",
    0x0012: "Purge-Jobs",
    0x0013: "Set-Printer-Attributes",
    0x0014: "Set-Job-Attributes",
    0x0015: "Get-Printer-Supported-Values",
    0x0016: "Create-Printer-Subscriptions",
    0x0017: "Create-Job-Subscriptions",
    0x0018: "Get-Subscription-Attributes",
    0x0019: "Get-Subscriptions",
    0x001A: "Renew-Subscription",
    0x001B: "Cancel-Subscription",
    0x001C: "Get-Notifications",
    0x001D: "ipp-indp-method",
    0x001E: "Get-Resource-Attributes",
    0x001F: "ipp-get-resources",
    0x0020: "Get-Resources",
    0x0021: "ipp-install",
    0x0022: "Enable-Printer",
    0x0023: "Disable-Printer",
    0x0024: "Pause-Printer-After-Current-Job",
    0x0025: "Hold-New-Jobs",
    0x0026: "Release-Held-New-Jobs",
    0x0027: "Deactivate-Printer",
    0x0028: "Activate-Printer",
    0x0029: "Restart-Printer",
    0x002A: "Shutdown-Printer",
    0x002B: "Startup-Printer",
    0x002C: "Reprocess-Job",
    0x002D: "Cancel-Current-Job",
    0x002E: "Suspend-Current-Job",
    0x002F: "Resume-Job",
    0x0030: "Promote-Job",
    0x0031: "Schedule-Job-After",
    0x0033: "Cancel-Document",
    0x0034: "Get-Document-Attributes",
    0x0035: "Get-Documents",
    0x0036: "Delete-Document",
    0x0037: "Set-Document-Attributes",
    0x0038: "Cancel-Jobs",
    0x0039: "Cancel-My-Jobs",
    0x003A: "Resubmit-Job",
    0x003B: "Close-Job",
    0x003C: "Identify-Printer",
    0x003D: "Validate-Document",
    0x003E: "Add-Document-Images",
    0x003F: "Acknowledge-Document",
    0x0040: "Acknowledge-Identify-Printer",
    0x0041: "Acknowledge-Job",
    0x0042: "Fetch-Document",
    0x0043: "Fetch-Job",
    0x0044: "Get-Output-Device-Attributes",
    0x0045: "Update-Active-Jobs",
    0x0046: "Deregister-Output-Device",
    0x0047: "Update-Document-Status",
    0x0048: "Update-Job-Status",
    0x0049: "Update-Output-Device-Attributes",
    0x004A: "Get-Next-Document-Data",
    0x004B: "Allocate-Printer-Resources",
    0x004C: "Create-Printer",
    0x004D: "Deallocate-Printer-Resources",
    0x004E: "Delete-Printer",
    0x004F: "Get-Printers",
    0x0050: "Shutdown-One-Printer",
    0x0051: "Startup-One-Printer",
    0x0052: "Cancel-Resource",
    0x0053: "Create-Resource",
    0x0054: "Install-Resource",
    0x0055: "Send-Resource-Data",
    0x0056: "Set-Resource-Attributes",
    0x0057: "Create-Resource-Subscriptions",
    0x0058: "Create-System-Subscriptions",
    0x0059: "Disable-All-Printers",
    0x005A: "Enable-All-Printers",
    0x005B: "Get-System-Attributes",
    0x005C: "Get-System-Supported-Values",
    0x005D: "Pause-All-Printers",
    0x005E: "Pause-All-Printers-After-Current-Job",
    0x005F: "Register-Output-Device",
    0x0060: "Restart-System",
    0x0061: "Resume-All-Printers",
    0x0062: "Set-System-Attributes",
    0x0063: "Shutdown-All-Printers",
    0x0064: "Startup-All-Printers",
    0x0065: "Get-Printer-Resources",
    0x0066: "Get-User-Printer-Attributes",
    0x0067: "Restart-One-Printer",
}
ATTRIBUTE_GROUP_TAGS = {
    0x00: "Reserved",
    0x01: "operation-attributes-tag",
    0x02: "job-attributes-tag",
    0x03: "end-of-attributes-tag",
    0x04: "printer-attributes-tag",
    0x05: "unsupported-attributes-tag",
    0x06: "subscription-attributes-tag",
    0x07: "event-notification-attributes-tag",
    0x08: "resource-attributes-tag",
    0x09: "document-attributes-tag",
    0x0A: "system-attributes-tag",
}
ATTRIBUTE_SYNTAXES = {
    0x20: "Unassigned",
    0x21: "integer",
    0x22: "boolean",
    0x23: "enum",
    0x30: "octetString",
    0x31: "dateTime",
    0x32: "resolution",
    0x33: "rangeOfInteger",
    0x34: "begCollection",
    0x35: "textWithLanguage",
    0x36: "nameWithLanguage",
    0x37: "endCollection",
    0x40: "Unassigned",
    0x41: "textWithoutLanguage",
    0x42: "nameWithoutLanguage",
    0x43: "Unassigned",
    0x44: "keyword",
    0x45: "uri",
    0x46: "uriScheme",
    0x47: "charset",
    0x48: "naturalLanguage",
    0x49: "mimeMediaType",
    0x4A: "memberAttrName",
    0x7F: "extension",
}
ATTRIBUTE_NAME_TO_VALUE_TAG = {
    "attributes-charset": b"\x47",
    "attributes-natural-language": b"\x48",
}
STATUS_CODE_OK = b"\x00\x00"
STATUS_CODE_BAD_REQUEST = b"\x04\x00"


class QIPPServer(BaseServer):
    NAME = "ipp_server"
    DEFAULT_PORT = 631

    def server_main(self):  # noqa: C901
        _q_s = self

        class MainResource(Resource):
            isLeaf = True  # noqa: N815

            def render_POST(self, request):  # noqa: N802
                client_ip, headers = get_headers_and_ip_from_request(request, _q_s.options)

                log_data = {
                    "action": "connection",
                    "src_ip": client_ip,
                    "src_port": request.getClientAddress().port,
                }
                if "capture_commands" in _q_s.options:
                    log_data["data"] = headers
                _q_s.log(log_data)

                data = request.content.read()

                response, status = self._build_response(data)

                if len(response) > 0:
                    _q_s.log(
                        {
                            "action": "query",
                            "status": status,
                            "src_ip": client_ip,
                            "src_port": request.getClientAddress().port,
                            "data": {"request": response},
                        }
                    )
                return self.send_response(data, status != "failed")

            def _build_response(self, data: bytes) -> tuple[str, str]:
                status = "success"
                version = [0, 0]
                groups = []

                try:
                    index, version[0] = get_uint8_t(0, data)
                    index, version[1] = get_uint8_t(index, data)
                    index, uint16_t = get_uint16_t(index, data)
                    operation = SUPPORTED_OPERATIONS[uint16_t]
                    index, request_id = get_uint32_t(index, data)
                    index, uint8_t = get_uint8_t(index, data)
                    group = ATTRIBUTE_GROUP_TAGS[uint8_t]
                    index, uint8_t = get_uint8_t(index, data)
                    if uint8_t in ATTRIBUTE_SYNTAXES:
                        groups, status = self._parse_attributes(data, index, uint8_t)

                    response = (
                        f"VERSION {version[0]}.{version[1]}|"
                        f"REQUEST {hex(request_id)}|"
                        f"OPERATION {operation}|"
                        f"GROUP {group}|"
                    )
                    if len(groups) > 0:
                        for attribute, values in groups:
                            response += f"ATTR {attribute} {','.join(values)}|"
                    with suppress(IndexError):
                        if response[-1] == "|":
                            response = response[:-1]
                except Exception as error:
                    _q_s.logger.debug(
                        f"[{_q_s.NAME}]: An error occurred during data parsing: {error}",
                        exc_info=True,
                    )
                    response = ""
                return response, status

            @staticmethod
            def _parse_attributes(
                data: bytes, index: int, attr_type: int
            ) -> tuple[list[tuple[str, list[str]]], str]:
                status = "success"
                groups = []
                to_parse_len = len(data[index:])
                while index < to_parse_len:
                    try:
                        value = ""
                        if ATTRIBUTE_SYNTAXES[attr_type] == "integer":
                            index, attribute = get_uint32_t(index, data)
                        elif ATTRIBUTE_SYNTAXES[attr_type] == "boolean":
                            index, attribute = get_uint8_t(index, data)
                        else:
                            index, uint16_t = get_uint16_t(index, data)
                            index, attribute = get_string(index, uint16_t, data)
                            index, uint16_t = get_uint16_t(index, data)
                            index, value = get_string(index, uint16_t, data)
                        if attribute == b"":
                            groups[-1][1].append(check_bytes(value))
                        else:
                            groups.append((check_bytes(attribute), [check_bytes(value)]))
                        index, attr_type = get_uint8_t(index, data)

                        if attr_type in ATTRIBUTE_GROUP_TAGS:
                            break
                    except (KeyError, IndexError, struct.error) as error:
                        _q_s.logger.debug(
                            f"[{_q_s.NAME}]: Error while parsing attributes: {error}",
                            exc_info=True,
                        )
                        status = "failed"
                        break
                return groups, status

            @staticmethod
            def send_response(request: bytes, successful: bool) -> bytes:
                version, request_id = request[0:2], request[3:7]
                if version not in [b"\x01\x01", b"\x02\x00", b"\x02\x01", b"\x02\x02"]:
                    version = b"\x02\x00"
                status_code = STATUS_CODE_OK if successful else STATUS_CODE_BAD_REQUEST
                attributes = attributes_dict_to_bytes(
                    {"attributes-charset": "utf-8", "attributes-natural-language": "en-us"}
                )
                return version + status_code + request_id + attributes

        reactor.listenTCP(self.port, Site(MainResource()))
        reactor.run()

    def test_server(self, ip=None, port=None):
        from socket import socket, AF_INET, SOCK_STREAM

        _ip = ip or self.ip
        _port = port or self.port

        body = (
            b"\x02\x00\x00\x0b\x00\x01/p\x01G\x00\x12attributes-charset\x00\x05utf-8H\x00\x1b"
            b"attributes-natural-language\x00\x02enE\x00\x0bprinter-uri\x00\x15"
            b"ipp://127.0.0.1:631/D\x00\x14requested-attributes\x00\x03allD\x00\x00\x00\x12"
            b"media-col-database\x03"
        )

        headers = (
            "POST / HTTP/1.1\r\n"
            "Content-Type: application/x-www-form-urlencoded\r\n"
            "Content-Length: {len(body)}\r\n"
            f"Host: {_ip}:{_port}\r\n"
            "Connection: close\r\n"
            "\r\n"
        )

        s = socket(AF_INET, SOCK_STREAM)
        s.connect((_ip, _port))
        s.sendall(headers.encode() + body)


def get_uint8_t(index, data):
    return index + 1, struct.unpack("b", data[index : index + 1])[0]


def get_uint16_t(index, data):
    return index + 2, struct.unpack(">H", data[index : index + 2])[0]


def get_uint32_t(index, data):
    return index + 4, struct.unpack(">I", data[index : index + 4])[0]


def get_string(index, length, data):
    return index + length, data[index : index + length]


def attributes_dict_to_bytes(attributes: dict[str, str]) -> bytes:
    attributes_str = b"\x01"  # start operation attributes
    for key, value in attributes.items():
        value_tag = ATTRIBUTE_NAME_TO_VALUE_TAG[key]
        name_length = len(key).to_bytes(2, "big")
        value_length = len(value).to_bytes(2, "big")
        attributes_str += value_tag + name_length + key.encode() + value_length + value.encode()
    attributes_str += b"\x03"  # end operation attributes
    return attributes_str


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qippserver = QIPPServer(
            ip=parsed.ip, port=parsed.port, options=parsed.options, config=parsed.config
        )
        qippserver.run_server()



from __future__ import annotations

import pytest

from honeypots import QSNMPServer
from .utils import (
    assert_connect_is_logged,
    IP,
    connect_to,
    load_logs_from_file,
    wait_for_server,
)

PORT = "50161"

@pytest.mark.parametrize(
    "server_logs",
    [{"server": QSNMPServer, "port": PORT}],
    indirect=True,
)
def test_postgres_server(server_logs):
    with wait_for_server(PORT), connect_to(IP, PORT, udp=True) as connection:
        connection.send(b'\x30\x2e\x02\x01\x01\x04\x06\x70\x75\x62\x6c\x69\x63\xa0\x21\x02\x04\x00\xab\x61\x67\x02\x01\x00\x02\x01\x00\x30\x13\x30\x11\x06\x0d\x2b\x06\x01\x04\x01\x09\x09\x84\x6a\x01\x04\x01\x00\x05\x00')

    logs = load_logs_from_file(server_logs)

    assert len(logs) >= 2
    connect, query, *_ = logs
    assert_connect_is_logged(connect, PORT)

    assert query["action"] == "query"
    assert query["data"] == {
        "community": "public",
        "oids": "1.3.6.1.4.1.9.9.618.1.4.1.0",
        "version": "1",
    }



from __future__ import annotations

from contextlib import suppress
from pathlib import Path

from Crypto.Cipher import DES
from twisted.internet import reactor
from twisted.internet.protocol import Factory, Protocol

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    check_bytes,
)


class QVNCServer(BaseServer):
    NAME = "vnc_server"
    DEFAULT_PORT = 5900

    def __init__(self, **kwargs):
        self.file_name = None
        super().__init__(**kwargs)
        self.challenge = bytes.fromhex("00000000901234567890123456789012")
        self.words = ["test", "admin", "123", "123456"]
        if self.file_name:
            self.load_words()
        self.known_passwords = {self.encode(w): w for w in [*self.words, self.password]}

    def load_words(self):
        path = Path(self.file_name)
        if not path.is_file():
            self.logger.error(f"[{self.NAME}]: Could not load word file: {path}")
            return
        self.words = path.read_text().splitlines()

    def encode(self, word: str) -> bytes:
        temp = word.strip("\n").ljust(8, "\00")[:8]
        rev_word = []
        for i in range(8):
            rev_word.append(int(f"{ord(temp[i]):08b}"[::-1], 2))
        return DES.new(bytes(rev_word), DES.MODE_ECB).encrypt(self.challenge)

    def decode(self, encoded_pw: bytes) -> str | None:
        return self.known_passwords.get(encoded_pw)

    def server_main(self):  # noqa: C901
        _q_s = self

        class CustomVNCProtocol(Protocol):
            _state = None

            def connectionMade(self):  # noqa: N802
                self.transport.write(b"RFB 003.008\n")
                self._state = 1
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )

            def dataReceived(self, data: bytes):  # noqa: N802
                if self._state == 1:
                    if data == b"RFB 003.008\n":
                        self._state = 2
                        self.transport.write(bytes.fromhex("0102"))
                elif self._state == 2:  # noqa: PLR2004
                    if data == b"\x02":
                        self._state = 3
                        self.transport.write(_q_s.challenge)
                elif self._state == 3:  # noqa: PLR2004
                    self._handle_login(data)
                    self.transport.loseConnection()
                else:
                    self.transport.loseConnection()

            def _handle_login(self, data: bytes):
                username = ""  # there is no user auth
                password = check_bytes(_q_s.decode(data))
                status = "success" if password == _q_s.password else "failed"
                if password is None:
                    password = f"DES:{data.hex()}"
                _q_s.log(
                    {
                        "action": "login",
                        "status": status,
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                        "username": username,
                        "password": password,
                    }
                )

            def connectionLost(self, reason):  # noqa: N802,ARG002
                self._state = None

        factory = Factory()
        factory.protocol = CustomVNCProtocol
        reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):
        from vncdotool import api

        with suppress(Exception):
            client = api.connect(
                f"{ip or self.ip}::{port or self.port}",
                username=username or self.username,
                password=password or self.password,
            )
            client.captureScreen("screenshot.png")
            client.disconnect()


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qvncserver = QVNCServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        qvncserver.run_server()



from __future__ import annotations

import pytest
from impacket.smbconnection import SMBConnection

from honeypots import QSMBServer
from .utils import (
    assert_connect_is_logged,
    IP,
    load_logs_from_file,
    PASSWORD,
    USERNAME,
    wait_for_server,
)

PORT = "50445"


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QSMBServer, "port": PORT}],
    indirect=True,
)
def test_smb_server(server_logs):
    with wait_for_server(PORT):
        smb_client = SMBConnection(IP, IP, sess_port=PORT)
        smb_client.login(USERNAME, PASSWORD)
        smb_client.close()

    *connects, login = load_logs_from_file(server_logs)

    assert len(connects) == 3
    for entry in connects:
        assert_connect_is_logged(entry, PORT)

    assert "Incoming connection" in connects[0]["data"]
    assert "AUTHENTICATE_MESSAGE" in connects[1]["data"]
    assert "authenticated successfully" in connects[2]["data"]

    assert login["action"] == "login"
    assert login["username"] == USERNAME



from __future__ import annotations

from abc import ABC
from cgi import FieldStorage
from contextlib import suppress
from random import choice

from twisted.web.resource import Resource

from honeypots.base_server import BaseServer
from honeypots.helper import load_template, get_headers_and_ip_from_request, check_bytes


class BaseHttpServer(BaseServer, ABC):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.mocking_server = choice(
            [
                "Apache",
                "nginx",
                "Microsoft-IIS/7.5",
                "Microsoft-HTTPAPI/2.0",
                "Apache/2.2.15",
                "SmartXFilter",
                "Microsoft-IIS/8.5",
                "Apache/2.4.6",
                "Apache-Coyote/1.1",
                "Microsoft-IIS/7.0",
                "Apache/2.4.18",
                "AkamaiGHost",
                "Apache/2.2.25",
                "Microsoft-IIS/10.0",
                "Apache/2.2.3",
                "nginx/1.12.1",
                "Apache/2.4.29",
                "cloudflare",
                "Apache/2.2.22",
            ]
        )

    class MainResource(Resource):
        isLeaf = True  # noqa: N815
        home_file = load_template("home.html")
        login_file = load_template("login.html")

        def __init__(self, *args, hp_server: BaseHttpServer = None, **kwargs):
            super().__init__(*args, **kwargs)
            self.hp_server = hp_server
            self.headers = {}

        def render(self, request):
            client_ip, headers = get_headers_and_ip_from_request(request, self.hp_server.options)

            with suppress(Exception):
                log_data = {
                    "action": "connection",
                    "src_ip": client_ip,
                    "src_port": request.getClientAddress().port,
                }
                if "capture_commands" in self.hp_server.options:
                    log_data["data"] = headers
                self.hp_server.log(log_data)

            if self.hp_server.mocking_server != "":
                request.responseHeaders.removeHeader("Server")
                request.responseHeaders.addRawHeader("Server", self.hp_server.mocking_server)

            if request.method in (b"GET", b"POST"):
                self.hp_server.log(
                    {
                        "action": request.method.decode(),
                        "src_ip": client_ip,
                        "src_port": request.getClientAddress().port,
                    }
                )

            if request.method == b"GET":
                if (
                    request.uri == b"/login.html"
                    and self.hp_server.username != ""
                    and self.hp_server.password != ""
                ):
                    request.responseHeaders.addRawHeader(
                        "Content-Type", "text/html; charset=utf-8"
                    )
                    return self.login_file

                request.responseHeaders.addRawHeader("Content-Type", "text/html; charset=utf-8")
                return self.login_file

            if request.method == b"POST":
                self.headers = request.getAllHeaders()
                if (
                    request.uri in (b"/login.html", b"/")
                    and self.hp_server.username != ""
                    and self.hp_server.password != ""
                ):
                    form = FieldStorage(
                        fp=request.content,
                        headers=self.headers,
                        environ={
                            "REQUEST_METHOD": "POST",
                            "CONTENT_TYPE": self.headers.get(
                                b"content-type",
                                b"application/x-www-form-urlencoded",
                            ),
                        },
                    )
                    if "username" in form and "password" in form:
                        username = check_bytes(form["username"].value)
                        password = check_bytes(form["password"].value)
                        self.hp_server.check_login(
                            username, password, client_ip, request.getClientAddress().port
                        )

            request.responseHeaders.addRawHeader("Content-Type", "text/html; charset=utf-8")
            return self.home_file



from __future__ import annotations

import socket
from base64 import b64decode
from contextlib import suppress
from twisted.internet import reactor
from twisted.internet.protocol import Factory
from twisted.mail.smtp import ESMTP
from honeypots.base_server import BaseServer
from honeypots.helper import check_bytes, server_arguments

class QSMTPServer(BaseServer):
    NAME = "smtp_server"
    DEFAULT_PORT = 25

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def server_main(self):  # noqa: C901
        _q_s = self

        class CustomSMTPProtocol(ESMTP):
            def __init__(self, *args, **kwargs):
                fun = None
                try:
                    # don't leak the *actual* hostname
                    fun = socket.getfqdn
                    socket.getfqdn = lambda: "ip-127-0-0-1.ec2.internal"
                    super().__init__(*args, **kwargs)
                finally:
                    if fun:
                        socket.getfqdn = fun

            def connectionMade(self):  # noqa: N802
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )
                super().connectionMade()

            def state_COMMAND(self, line):
                command, *rest = line.split(b" ")
                arg = rest[0] if rest else None
                data = rest[1] if len(rest) > 1 else None
                if command.upper() not in {b"HELO", b"EHLO"}:
                    _q_s.log(
                        {
                            "action": "connection",
                            "src_ip": self.transport.getPeer().host,
                            "src_port": self.transport.getPeer().port,
                            "data": {"command": command, "arg": arg, "data": data},
                        }
                    )
                super().state_COMMAND(line)

            def do_EHLO(self, arg):
                self.sendCode(250, f"ip-127-0-0-1.ec2.internal Hello {arg}\n8BITMIME\nAUTH LOGIN PLAIN\nSTARTTLS".encode())

            def ext_AUTH(self, arg):
                if arg.startswith(b"PLAIN "):
                    _, username, password = (
                        b64decode(arg.split(b" ")[1].strip())
                        .decode("utf-8", errors="replace")
                        .split("\0")
                    )
                    _q_s.check_login(username, password, self.transport.getPeer().host, self.transport.getPeer().port)
                self.sendCode(235,b'Authentication successful.')

        class CustomSMTPFactory(Factory):
            protocol = CustomSMTPProtocol
            portal = None

            def buildProtocol(self, address):  # noqa: N802,ARG002
                p = self.protocol()
                p.portal = self.portal
                p.factory = self
                return p

        factory = CustomSMTPFactory()
        reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from smtplib import SMTP

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password
            s = SMTP(_ip, _port)
            s.ehlo()
            s.login(_username, _password)
            s.quit()


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        QSMTPserver = QSMTPServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        QSMTPserver.run_server()



from __future__ import annotations

import math
from struct import unpack
from time import time

import pytest

from honeypots import QNTPServer
from .utils import (
    connect_to,
    IP,
    load_logs_from_file,
    wait_for_server,
)

PORT = "50123"


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QNTPServer, "port": PORT}],
    indirect=True,
)
def test_ntp_server(server_logs):
    with wait_for_server(PORT), connect_to(IP, PORT, udp=True) as connection:
        connection.send(b"\x1b" + 47 * b"\0")
        data, _ = connection.recvfrom(256)
        output_time = unpack("!12I", data)[10] - 2208988800

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 2
    connect, query = logs
    assert all(k in connect for k in ["action", "server", "src_ip", "src_port", "timestamp"])
    assert connect["action"] == "connection"
    assert connect["src_ip"] == IP

    assert query["action"] == "query"
    assert query["status"] == "success"
    assert query["data"] == {"mode": "3", "version": "3"}

    assert math.isclose(output_time, time(), abs_tol=10)



from __future__ import annotations

from pathlib import Path

import pytest
import requests

from honeypots import QHTTPProxyServer
from .utils import (
    assert_connect_is_logged,
    IP,
    load_logs_from_file,
    wait_for_server,
)

PORT = "58080"
PORT_2 = "58081"
SERVER_CONFIG = {
    "honeypots": {
        "httpproxy": {
            "options": ["capture_commands"],
        },
    }
}


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QHTTPProxyServer, "port": PORT, "custom_config": SERVER_CONFIG}],
    indirect=True,
)
def test_http_proxy_server(server_logs):
    with wait_for_server(PORT):
        response = requests.get(
            "http://example.com/",
            proxies={"http": f"http://{IP}:{PORT}"},
            timeout=2,
        )

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 2
    connect, query = logs
    assert_connect_is_logged(connect, PORT)

    assert query["data"] == "example.com"
    assert query["action"] == "query"

    assert response.ok
    assert "Example Website" in response.text, "dummy response is missing"


CUSTOM_TEMPLATE = Path(__file__).parent / "data" / "test_template.html"
CUSTOM_TEMPLATE_CONFIG = {
    "honeypots": {
        "httpproxy": {
            "template": str(CUSTOM_TEMPLATE.absolute()),
        },
    }
}


@pytest.mark.parametrize(
    "server_logs",
    [
        {
            "server": QHTTPProxyServer,
            "port": PORT_2,
            "custom_config": CUSTOM_TEMPLATE_CONFIG,
        }
    ],
    indirect=True,
)
def test_custom_template(server_logs):  # noqa: ARG001
    with wait_for_server(PORT_2):
        response = requests.get(
            "http://example.com/",
            proxies={"http": f"http://{IP}:{PORT_2}"},
            timeout=2,
        )

    assert response.ok
    assert "This is a template for testing!" in response.text, "custom template is missing"



from __future__ import annotations

from contextlib import suppress

from twisted.internet import defer, reactor
from twisted.names import dns, error, client
from twisted.names.server import DNSServerFactory

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
)


class QDNSServer(BaseServer):
    NAME = "dns_server"
    DEFAULT_PORT = 53

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.resolver_addresses = [("8.8.8.8", 53)]

    def server_main(self):
        _q_s = self

        class CustomClientResolver(client.Resolver):
            def queryUDP(self, queries, timeout=2):  # noqa: N802
                res = client.Resolver.queryUDP(self, queries, timeout)

                def queryFailed(reason):  # noqa: N802,ARG001
                    return defer.fail(error.DomainError())

                res.addErrback(queryFailed)
                return res

        class CustomDNSServerFactory(DNSServerFactory):
            def gotResolverResponse(self, response, protocol, message, address):  # noqa: N802
                if address is None:
                    src_ip, src_port = "None", "None"
                else:
                    src_ip, src_port = address
                for items in response:
                    for item in items:
                        _q_s.log(
                            {
                                "action": "query",
                                "src_ip": src_ip,
                                "src_port": src_port,
                                "data": item.payload,
                            }
                        )
                return super().gotResolverResponse(response, protocol, message, address)

        class CustomDnsUdpProtocol(dns.DNSDatagramProtocol):
            def datagramReceived(self, data: bytes, addr: tuple[str, int]):  # noqa: N802
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": addr[0],
                        "src_port": addr[1],
                        "data": data.decode(errors="replace"),
                    }
                )
                super().datagramReceived(data, addr)

        self.resolver = CustomClientResolver(servers=self.resolver_addresses)
        self.factory = CustomDNSServerFactory(clients=[self.resolver])
        self.protocol = CustomDnsUdpProtocol(controller=self.factory)
        reactor.listenUDP(self.port, self.protocol, interface=self.ip)
        reactor.listenTCP(self.port, self.factory, interface=self.ip)
        reactor.run()

    def test_server(self, *_, domain=None, **__):
        with suppress(Exception):
            from dns.resolver import Resolver

            res = Resolver(configure=False)
            res.nameservers = [self.ip]
            res.port = self.port
            temp_domain = domain or "example.org"
            res.resolve(temp_domain, "a")


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qdnsserver = QDNSServer(ip=parsed.ip, port=parsed.port, config=parsed.config)
        qdnsserver.run_server()
